import csv
from datetime import timedelta
from functools import partial
import os
from pubsub import pub
from threading import Lock, Thread
from time import localtime, sleep, time
import wx
from wx.lib.filebrowsebutton import DirBrowseButton

from spacq.interface.pulse.parser import PulseError
from spacq.iteration.sweep import PulseConfiguration, SweepController
from spacq.iteration.variables import sort_variables, InputVariable, OutputVariable
from spacq.tool.box import flatten, sift

from ..tool.box import Dialog, MessageDialog, YesNoQuestionDialog


class DataCaptureDialog(Dialog, SweepController):
	"""
	A progress dialog which runs over iterators, sets the corresponding resources, and captures the measured data.
	"""

	max_value_len = 50 # characters

	timer_delay = 50 # ms
	stall_time = 2 # s

	status_messages = {
		None: 'Starting up',
		'init': 'Initializing',
		'next': 'Getting next values',
		'transition': 'Smooth setting',
		'write': 'Writing to devices',
		'dwell': 'Waiting for resources to settle',
		'pulse': 'Running pulse program',
		'read': 'Taking measurements',
		'ramp_down': 'Smooth setting',
		'end': 'Finishing',
	}

	def __init__(self, parent, resources, variables, num_items, measurement_resources,
			measurement_variables, pulse_config, continuous=False,
			*args, **kwargs):
		kwargs['style'] = kwargs.get('style', wx.DEFAULT_DIALOG_STYLE) | wx.RESIZE_BORDER

		Dialog.__init__(self, parent, title='Sweeping...', *args, **kwargs)
		SweepController.__init__(self, resources, variables, num_items, measurement_resources,
				measurement_variables, pulse_config, continuous=continuous)

		self.parent = parent

		# Show only elapsed time in continuous mode.
		self.show_remaining_time = not self.continuous

		self.last_checked_time = -1
		self.elapsed_time = 0 # us

		self.timer = wx.Timer(self)
		self.Bind(wx.EVT_TIMER, self.OnTimer, self.timer)

		self.cancelling = False

		def write_callback(pos, i, value):
			self.value_outputs[pos][i].Value = str(value)[:self.max_value_len]
		self.write_callback = partial(wx.CallAfter, write_callback)

		def read_callback(i, value):
			self.value_inputs[i].Value = str(value)[:self.max_value_len]
		self.read_callback = partial(wx.CallAfter, read_callback)

		self.general_exception_handler = partial(wx.CallAfter, self._general_exception_handler)
		self.resource_exception_handler = partial(wx.CallAfter, self._resource_exception_handler)

		# Dialog.
		dialog_box = wx.BoxSizer(wx.VERTICAL)

		## Progress.
		progress_box = wx.BoxSizer(wx.HORIZONTAL)
		dialog_box.Add(progress_box, flag=wx.EXPAND|wx.ALL, border=5)

		### Message.
		self.progress_percent = wx.StaticText(self, label='', size=(40, -1))
		progress_box.Add(self.progress_percent,
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT|wx.ALL, border=5)

		### Bar.
		self.progress_bar = wx.Gauge(self, range=num_items, style=wx.GA_HORIZONTAL)
		progress_box.Add(self.progress_bar, proportion=1)

		## Status.
		self.status_message_output = wx.TextCtrl(self, style=wx.TE_READONLY)
		self.status_message_output.BackgroundColour = wx.LIGHT_GREY
		dialog_box.Add(self.status_message_output, flag=wx.EXPAND)

		## Values.
		self.values_box = wx.FlexGridSizer(rows=len(self.variables), cols=2, hgap=20)
		self.values_box.AddGrowableCol(1, 1)
		dialog_box.Add(self.values_box, flag=wx.EXPAND|wx.ALL, border=5)

		self.value_outputs = []
		for group in self.variables:
			group_outputs = []

			for var in group:
				output = wx.TextCtrl(self, style=wx.TE_READONLY)
				output.BackgroundColour = wx.LIGHT_GREY
				group_outputs.append(output)

				self.values_box.Add(wx.StaticText(self, label=var.name),
						flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
				self.values_box.Add(output, flag=wx.EXPAND)

			self.value_outputs.append(group_outputs)

			# Spacer.
			for _ in xrange(2):
				self.values_box.Add((-1, 15))

		# Separator.
		for _ in xrange(2):
			self.values_box.Add(wx.StaticLine(self), flag=wx.EXPAND|wx.ALL, border=5)

		self.value_inputs = []
		for var in self.measurement_variables:
			input = wx.TextCtrl(self, style=wx.TE_READONLY)
			input.BackgroundColour = wx.LIGHT_GREY
			self.value_inputs.append(input)

			self.values_box.Add(wx.StaticText(self, label=var.name),
					flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
			self.values_box.Add(input, flag=wx.EXPAND)

		## Times.
		times_box = wx.FlexGridSizer(rows=2 if self.show_remaining_time else 1, cols=2, hgap=5)
		dialog_box.Add(times_box, proportion=1, flag=wx.CENTER|wx.ALL, border=15)

		### Elapsed.
		times_box.Add(wx.StaticText(self, label='Elapsed time:'))
		self.elapsed_time_output = wx.StaticText(self, label='---:--:--')
		times_box.Add(self.elapsed_time_output)

		### Remaining.
		if self.show_remaining_time:
			times_box.Add(wx.StaticText(self, label='Remaining time:'))
			self.remaining_time_output = wx.StaticText(self, label='---:--:--')
			times_box.Add(self.remaining_time_output)

		## Last continuous.
		if self.continuous:
			self.last_continuous_input = wx.CheckBox(self, label='Last loop of continuous sweep')
			dialog_box.Add(self.last_continuous_input, flag=wx.CENTER)

		## End button.
		button_box = wx.BoxSizer(wx.HORIZONTAL)
		dialog_box.Add(button_box, flag=wx.CENTER)

		self.cancel_button = wx.Button(self, label='Cancel')
		self.Bind(wx.EVT_BUTTON, self.OnCancel, self.cancel_button)
		button_box.Add(self.cancel_button)

		self.SetSizerAndFit(dialog_box)

		# Try to cancel cleanly instead of giving up.
		self.Bind(wx.EVT_CLOSE, self.OnCancel)

	def _general_exception_handler(self, f, e):
		"""
		Called when a trampolined function raises e.
		"""

		MessageDialog(self.parent, '{0}'.format(str(e)), 'Sweep error in "{0}"'.format(f)).Show()

	def _resource_exception_handler(self, resource_name, e, write=True):
		"""
		Called when a write to or read from a Resource raises e.
		"""

		msg = 'Resource: {0}\nError: {1}'.format(resource_name, str(e))
		dir = 'writing to' if write else 'reading from'
		MessageDialog(self.parent, msg, 'Error {0} resource'.format(dir)).Show()

		self.abort(fatal=write)

	def start(self):
		thr = Thread(target=SweepController.run, args=(self,))
		thr.daemon = True
		thr.start()

		self.timer.Start(self.timer_delay)

	def dwell(self):
		result = SweepController.dwell(self)

		# Prevent the GUI from locking up.
		sleep(0.005)

		return result

	def end(self):
		try:
			SweepController.end(self)
		except AssertionError:
			return

		# In case the sweep is too fast, ensure that the user has some time to see the dialog.
		span = time() - self.sweep_start_time
		if span < self.stall_time:
			sleep(self.stall_time - span)

		wx.CallAfter(self.timer.Stop)
		wx.CallAfter(self.Destroy)

	def OnCancel(self, evt=None):
		if not self.cancel_button.Enabled:
			return

		self.cancel_button.Disable()
		self.cancelling = True

	def OnTimer(self, evt=None):
		self.status_message_output.Value = self.status_messages[self.current_f]
		if self.continuous:
			self.last_continuous = self.last_continuous_input.Value

		# Update progress.
		if self.num_items > 0 and self.item >= 0:
			amount_done = float(self.item) / self.num_items

			self.progress_bar.Value = self.item
			self.progress_percent.Label = '{0}%'.format(int(100 * amount_done))

			if self.last_checked_time > 0:
				self.elapsed_time += int((time() - self.last_checked_time) * 1e6)
				self.elapsed_time_output.Label = str(timedelta(seconds=self.elapsed_time//1e6))

			self.last_checked_time = time()

			if self.show_remaining_time and amount_done > 0:
				total_time = self.elapsed_time / amount_done
				remaining_time = int(total_time - self.elapsed_time)
				self.remaining_time_output.Label = str(timedelta(seconds=remaining_time//1e6))

		# Prompt to abort.
		if self.cancelling:
			def abort():
				self.cancelling = False

				thr = Thread(target=self.abort)
				thr.daemon = True
				thr.start()

				self.timer.Start(self.timer_delay)

			def resume():
				self.cancelling = False
				self.cancel_button.Enable()

				self.unpause()

				self.timer.Start(self.timer_delay)

			self.pause()

			self.last_checked_time = -1
			self.timer.Stop()

			YesNoQuestionDialog(self, 'Abort processing?', abort, resume).Show()

			return


class DataCapturePanel(wx.Panel):
	"""
	A panel to start the data capture process, optionally exporting the results to a file.
	"""

	def __init__(self, parent, global_store, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		self.global_store = global_store

		self.capture_dialogs = 0

		# Panel.
		panel_box = wx.BoxSizer(wx.HORIZONTAL)

		## Capture.
		capture_static_box = wx.StaticBox(self, label='Capture')
		capture_box = wx.StaticBoxSizer(capture_static_box, wx.VERTICAL)
		panel_box.Add(capture_box, flag=wx.CENTER|wx.ALL, border=5)

		### Start.
		self.start_button = wx.Button(self, label='Start')
		self.Bind(wx.EVT_BUTTON, self.OnBeginCapture, self.start_button)
		capture_box.Add(self.start_button, flag=wx.CENTER)

		### Continuous.
		self.continuous_checkbox = wx.CheckBox(self, label='Continuous')
		capture_box.Add(self.continuous_checkbox, flag=wx.CENTER)

		## Export.
		export_static_box = wx.StaticBox(self, label='Export')
		export_box = wx.StaticBoxSizer(export_static_box, wx.HORIZONTAL)
		panel_box.Add(export_box, proportion=1, flag=wx.CENTER|wx.ALL, border=5)

		### Enabled.
		self.export_enabled = wx.CheckBox(self, label='')
		self.export_enabled.Value = True
		export_box.Add(self.export_enabled, flag=wx.CENTER)

		### Export path.
		export_path_box = wx.BoxSizer(wx.VERTICAL)
		export_box.Add(export_path_box, proportion=1, flag=wx.CENTER)

		#### Directory.
		self.directory_browse_button = DirBrowseButton(self, labelText='Directory:')
		export_path_box.Add(self.directory_browse_button, flag=wx.EXPAND)

		#### Last file.
		last_file_box = wx.BoxSizer(wx.HORIZONTAL)
		export_path_box.Add(last_file_box, flag=wx.EXPAND)

		last_file_box.Add(wx.StaticText(self, label='Last output: '),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.last_file_name = wx.TextCtrl(self, style=wx.TE_READONLY)
		self.last_file_name.BackgroundColour = wx.LIGHT_GREY
		last_file_box.Add(self.last_file_name, proportion=1)

		self.SetSizer(panel_box)

	def OnBeginCapture(self, evt=None):
		# Prevent accidental double-clicking.
		self.start_button.Disable()
		def enable_button():
			sleep(1)
			wx.CallAfter(self.start_button.Enable)
		thr = Thread(target=enable_button)
		thr.daemon = True
		thr.start()

		all_variables = [var for var in self.global_store.variables.values() if var.enabled]
		output_variables = sift(all_variables, OutputVariable)
		input_variables = [var for var in sift(all_variables, InputVariable) if var.resource_name != '']

		if not output_variables:
			output_variables.append(OutputVariable(order=0, name='<Dummy>', enabled=True))

		output_variables, num_items = sort_variables(output_variables)

		resource_names = [tuple(var.resource_name for var in group) for group in output_variables]
		measurement_resource_names = [var.resource_name for var in input_variables]

		continuous = self.continuous_checkbox.Value

		missing_resources = set()
		unreadable_resources = set()
		unwritable_resources = set()
		missing_devices = set()

		pulse_program = self.global_store.pulse_program

		if pulse_program is not None:
			pulse_program = pulse_program.with_resources

			try:
				pulse_program.generate_waveforms(dry_run=True)
			except PulseError as e:
				MessageDialog(self, '\n'.join(e[0]), 'Pulse program error', monospace=True).Show()
				return
			except Exception as e:
				MessageDialog(self, str(e), 'Pulse program error').Show()
				return

			pulse_awg, pulse_oscilloscope = None, None
			pulse_channels = {}

			try:
				pulse_awg = self.global_store.devices[pulse_program.awg].device
				if pulse_awg is None:
					raise KeyError
			except KeyError:
				missing_devices.add(pulse_program.awg)
			else:
				# Gather used channel numbers.
				pulse_channels = dict((k, v) for k, v in pulse_program.output_channels.items() if v is not None)

				actual_channels = range(1, len(pulse_awg.channels))
				invalid_channels = [k for k, v in pulse_channels.items() if v not in actual_channels]

				if invalid_channels:
					MessageDialog(self, 'Invalid channels for: {0}'.format(', '.join(invalid_channels)), 'Invalid channels').Show()
					return

			try:
				pulse_oscilloscope = self.global_store.devices[pulse_program.oscilloscope].device
				if pulse_oscilloscope is None:
					raise KeyError
			except KeyError:
				missing_devices.add(pulse_program.oscilloscope)

			try:
				pulse_config = PulseConfiguration(pulse_program, pulse_channels, pulse_awg, pulse_oscilloscope)
			except TypeError as e:
				MessageDialog(self, str(e), 'Device configuration error').Show()
				return
		else:
			pulse_config = None

		resources = []
		for group in resource_names:
			group_resources = []

			for name in group:
				if name == '':
					group_resources.append((str(len(resources)), None))
				elif name not in self.global_store.resources:
					missing_resources.add(name)
				else:
					resource = self.global_store.resources[name]

					if resource.writable:
						group_resources.append((name, resource))
					else:
						unwritable_resources.add(name)

			resources.append(tuple(group_resources))

		measurement_resources = []
		measurement_units = []
		for name in measurement_resource_names:
			if name not in self.global_store.resources:
				missing_resources.add(name)
			else:
				resource = self.global_store.resources[name]

				if resource.readable:
					measurement_resources.append((name, resource))
					measurement_units.append(resource.display_units)
				else:
					unreadable_resources.add(name)

		mismatched_resources = []
		for (res_name, resource), var in zip(flatten(resources), flatten(output_variables)):
			if resource is None:
				continue

			if resource.units is not None:
				if not (var.type == 'quantity' and
						resource.verify_dimensions(var.units, exception=False, from_string=True)):
					mismatched_resources.append((res_name, var.name))
			else:
				if var.type not in ['float', 'integer']:
					mismatched_resources.append((res_name, var.name))

		for items, msg in [
			(missing_resources, 'Missing resources'),
			(unreadable_resources, 'Unreadable resources'),
			(unwritable_resources, 'Unwritable resources'),
			(missing_devices, 'Missing devices')]:

			if items:
				MessageDialog(self, ', '.join('"{0}"'.format(x) for x in sorted(items)), msg).Show()

		if mismatched_resources:
			MessageDialog(self, ', '.join('{0}/{1}'.format(x[0], x[1]) for x in mismatched_resources),
					'Mismatched resources').Show()

		if (missing_resources or unreadable_resources or unwritable_resources or
				missing_devices or mismatched_resources):
			return

		exporting = False
		if self.export_enabled.Value:
			dir = self.directory_browse_button.GetValue()
			# YYYY-MM-DD_HH-MM-SS.csv
			name = '{0:04}-{1:02}-{2:02}_{3:02}-{4:02}-{5:02}.csv'.format(*localtime())

			if not dir:
				MessageDialog(self, 'No directory selected.', 'Export path').Show()
				return

			if not os.path.isdir(dir):
				MessageDialog(self, 'Invalid directory selected', 'Export path').Show()
				return

			file_path = os.path.join(dir, name)
			if os.path.exists(file_path):
				MessageDialog(self, file_path, 'File exists').Show()
				return

			# Everything looks alright, so open the file.
			export_file = open(file_path, 'w')
			export_csv = csv.writer(export_file)
			exporting = True

			# Show the path in the GUI.
			self.last_file_name.Value = file_path

			# Write the header.
			export_csv.writerow(['Time (s)'] +
					['{0.name} ({0.units})'.format(var) if var.units is not None else var.name
							for var in flatten(output_variables)] +
					['{0.name} ({1})'.format(var, units) if units is not None else var.name
						for var, units in zip(input_variables, measurement_units)])

		self.capture_dialogs += 1

		dlg = DataCaptureDialog(self, resources, output_variables, num_items, measurement_resources,
				input_variables, pulse_config, continuous=continuous)
		dlg.SetMinSize((500, -1))

		for name in measurement_resource_names:
			pub.sendMessage('data_capture.start', name=name)

		# Export buffer.
		max_buf_size = 10
		buf = []
		buf_lock = Lock()

		def flush():
			export_csv.writerows(buf)
			export_file.flush()

			while buf:
				buf.pop()

		def data_callback(cur_time, values, measurement_values):
			for name, value in zip(measurement_resource_names, measurement_values):
				pub.sendMessage('data_capture.data', name=name, value=value)

			# Extract values out of quantities, since the units have already been taken care of in the header.
			values = [x.original_value if hasattr(x, 'original_value') else x for x in values]
			measurement_values = [x.original_value if hasattr(x, 'original_value') else x for x in measurement_values]

			if exporting:
				with buf_lock:
					buf.append([cur_time] + values + measurement_values)

					if len(buf) >= max_buf_size:
						flush()

		def close_callback():
			self.capture_dialogs -= 1

			if exporting:
				with buf_lock:
					flush()
					export_file.close()

			for name in measurement_resource_names:
				pub.sendMessage('data_capture.stop', name=name)

		dlg.data_callback = data_callback
		dlg.close_callback = close_callback
		dlg.Show()
		dlg.start()



from nose.tools import eq_
from unittest import main, TestCase

from .. import box


class DetermineWildcardTest(TestCase):
	def testDefault(self):
		"""
		Simply all files.
		"""

		w = box.determine_wildcard()
		eq_(w, 'All files|*')

	def testExtension(self):
		"""
		With a nameless extension.
		"""

		w = box.determine_wildcard('test')
		eq_(w, '(*.test)|*.test|All files|*')

	def testFileType(self):
		"""
		With a named extension.
		"""

		w = box.determine_wildcard('test', 'Test')
		eq_(w, 'Test (*.test)|*.test|All files|*')

	def testInvalid(self):
		"""
		Try some invalid combinations.
		"""

		# Unused argument.
		w = box.determine_wildcard(file_type='Test')
		eq_(w, 'All files|*')

		# Invalid characters.
		try:
			box.determine_wildcard('test|test', 'Test')
		except ValueError:
			pass
		else:
			assert False, 'Expected ValueError.'

		try:
			box.determine_wildcard('test', 'Test|Test')
		except ValueError:
			pass
		else:
			assert False, 'Expected ValueError.'


if __name__ == '__main__':
	main()



import ObjectListView
import wx

from spacq.interface.units import Quantity
from spacq.iteration.variables import OutputVariable, LinSpaceConfig, ArbitraryConfig

from ..tool.box import Dialog, MessageDialog, load_pickled, save_pickled

"""
An interface for creating and editing Variable objects.
"""


class VariableColumnDefn(ObjectListView.ColumnDefn):
	"""
	A column with useful defaults.
	"""

	def __init__(self, width=90, align='centre', groupKeyGetter='order', *args, **kwargs):
		ObjectListView.ColumnDefn.__init__(self, width=width, align=align,
				groupKeyGetter=groupKeyGetter, *args, **kwargs)

		# No auto-width if space filling.
		if self.isSpaceFilling:
			self.width = 0


class LinSpaceConfigPanel(wx.Panel):
	def __init__(self, parent, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		# Panel.
		panel_box = wx.BoxSizer(wx.VERTICAL)

		## Config.
		config_sizer = wx.FlexGridSizer(rows=3, cols=2)
		config_sizer.AddGrowableCol(1, 1)
		panel_box.Add(config_sizer, proportion=1, flag=wx.EXPAND)

		### Initial.
		config_sizer.Add(wx.StaticText(self, label='Initial:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT|wx.ALL, border=5)
		self.initial_input = wx.TextCtrl(self)
		config_sizer.Add(self.initial_input, flag=wx.EXPAND|wx.ALL, border=5)

		### Final.
		config_sizer.Add(wx.StaticText(self, label='Final:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT|wx.ALL, border=5)
		self.final_input = wx.TextCtrl(self)
		config_sizer.Add(self.final_input, flag=wx.EXPAND|wx.ALL, border=5)

		### Steps.
		config_sizer.Add(wx.StaticText(self, label='Steps:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT|wx.ALL, border=5)
		self.steps_input = wx.SpinCtrl(self, min=1, initial=1, max=1e9)
		config_sizer.Add(self.steps_input, flag=wx.EXPAND|wx.ALL, border=5)

		self.SetSizerAndFit(panel_box)

	def GetValue(self):
		# Ensure the values are sane.
		try:
			initial = float(self.initial_input.Value)
		except ValueError:
			raise ValueError('Invalid initial value.')

		try:
			final = float(self.final_input.Value)
		except ValueError:
			raise ValueError('Invalid final value.')

		return LinSpaceConfig(initial, final, self.steps_input.Value)

	def SetValue(self, config):
		self.initial_input.Value, self.final_input.Value, self.steps_input.Value = (str(config.initial),
				str(config.final), config.steps)


class ArbitraryConfigPanel(wx.Panel):
	def __init__(self, parent, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		# Panel.
		panel_box = wx.BoxSizer(wx.VERTICAL)

		## Config.
		config_sizer = wx.FlexGridSizer(rows=1, cols=2)
		config_sizer.AddGrowableCol(1, 1)
		panel_box.Add(config_sizer, proportion=1, flag=wx.EXPAND)

		### Values.
		config_sizer.Add(wx.StaticText(self, label='Values:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT|wx.ALL, border=5)
		self.values_input = wx.TextCtrl(self)
		config_sizer.Add(self.values_input, flag=wx.EXPAND|wx.ALL, border=5)

		self.SetSizerAndFit(panel_box)

	def GetValue(self):
		raw_values = self.values_input.Value.split(',')

		# Ensure the values are sane.
		try:
			values = [float(x) for x in raw_values]
		except ValueError as e:
			raise ValueError('Invalid value: {0}'.format(str(e)))

		return ArbitraryConfig(values)

	def SetValue(self, config):
		self.values_input.Value = ', '.join('{0:n}'.format(x) for x in config.values)


class VariableEditor(Dialog):
	def __init__(self, parent, ok_callback, *args, **kwargs):
		kwargs['style'] = kwargs.get('style', wx.DEFAULT_DIALOG_STYLE) | wx.RESIZE_BORDER

		Dialog.__init__(self, parent, *args, **kwargs)

		self.ok_callback = ok_callback

		# Dialog.
		dialog_box = wx.BoxSizer(wx.VERTICAL)

		## Config.
		self.config_notebook = wx.Notebook(self)
		dialog_box.Add(self.config_notebook, proportion=1, flag=wx.EXPAND|wx.ALL, border=5)

		self.config_panel_types = []

		### Linear.
		linspace_config_panel = LinSpaceConfigPanel(self.config_notebook)
		self.config_panel_types.append(LinSpaceConfig)
		self.config_notebook.AddPage(linspace_config_panel, 'Linear')

		### Arbitrary.
		arbitrary_config_panel = ArbitraryConfigPanel(self.config_notebook)
		self.config_panel_types.append(ArbitraryConfig)
		self.config_notebook.AddPage(arbitrary_config_panel, 'Arbitrary')

		## Smooth set.
		smooth_static_box = wx.StaticBox(self, label='Smooth set')
		smooth_box = wx.StaticBoxSizer(smooth_static_box, wx.HORIZONTAL)
		dialog_box.Add(smooth_box, flag=wx.CENTER|wx.ALL, border=5)

		smooth_box.Add(wx.StaticText(self, label='Steps:'), flag=wx.CENTER)

		self.smooth_steps_input = wx.SpinCtrl(self, min=1, initial=10)
		smooth_box.Add(self.smooth_steps_input, flag=wx.CENTER|wx.ALL, border=5)

		self.smooth_from_checkbox = wx.CheckBox(self, label='From const')
		smooth_box.Add(self.smooth_from_checkbox, flag=wx.CENTER|wx.ALL, border=5)

		self.smooth_to_checkbox = wx.CheckBox(self, label='To const')
		smooth_box.Add(self.smooth_to_checkbox, flag=wx.CENTER|wx.ALL, border=5)

		self.smooth_transition_checkbox = wx.CheckBox(self, label='Transition')
		smooth_box.Add(self.smooth_transition_checkbox, flag=wx.CENTER|wx.ALL, border=5)

		## Type.
		type_static_box = wx.StaticBox(self, label='Type')
		type_box = wx.StaticBoxSizer(type_static_box, wx.HORIZONTAL)
		dialog_box.Add(type_box, flag=wx.CENTER|wx.ALL, border=5)

		self.type_float = wx.RadioButton(self, label='Float', style=wx.RB_GROUP)
		type_box.Add(self.type_float, flag=wx.CENTER|wx.ALL, border=5)

		self.type_integer = wx.RadioButton(self, label='Integer')
		type_box.Add(self.type_integer, flag=wx.CENTER|wx.ALL, border=5)

		### Units.
		quantity_static_box = wx.StaticBox(self, label='Quantity')
		quantity_box = wx.StaticBoxSizer(quantity_static_box, wx.HORIZONTAL)
		type_box.Add(quantity_box)

		self.type_quantity = wx.RadioButton(self)
		quantity_box.Add(self.type_quantity, flag=wx.CENTER)

		self.units_input = wx.TextCtrl(self)
		quantity_box.Add(self.units_input)

		## End buttons.
		button_box = wx.BoxSizer(wx.HORIZONTAL)
		dialog_box.Add(button_box, flag=wx.CENTER|wx.ALL, border=5)

		ok_button = wx.Button(self, wx.ID_OK)
		self.Bind(wx.EVT_BUTTON, self.OnOk, ok_button)
		button_box.Add(ok_button)

		cancel_button = wx.Button(self, wx.ID_CANCEL)
		button_box.Add(cancel_button)

		self.SetSizerAndFit(dialog_box)

	def GetValue(self):
		if self.type_float.Value:
			type = 'float'
			units = None
		elif self.type_integer.Value:
			type = 'integer'
			units = None
		else:
			type = 'quantity'
			units = self.units_input.Value

			# Ensure that the units are valid.
			Quantity(1, units)

		return (self.config_notebook.CurrentPage.GetValue(), self.smooth_steps_input.Value,
				self.smooth_from_checkbox.Value, self.smooth_to_checkbox.Value,
				self.smooth_transition_checkbox.Value, type, units)

	def SetValue(self, config, smooth_steps, smooth_from, smooth_to, smooth_transition, type, units):
		config_type = self.config_panel_types.index(config.__class__)
		self.config_notebook.ChangeSelection(config_type)
		self.config_notebook.CurrentPage.SetValue(config)

		(self.smooth_steps_input.Value, self.smooth_from_checkbox.Value,
				self.smooth_to_checkbox.Value,
				self.smooth_transition_checkbox.Value) = smooth_steps, smooth_from, smooth_to, smooth_transition

		if type == 'float':
			self.type_float.Value = True
		elif type == 'integer':
			self.type_integer.Value = True
		else:
			self.type_quantity.Value = True
			self.units_input.Value = units if units is not None else ''

	def OnOk(self, evt=None):
		if self.ok_callback(self):
			self.Destroy()


class VariablesPanel(wx.Panel):
	col_name = VariableColumnDefn(checkStateGetter='enabled', title='Name', valueGetter='name',
			width=150, align='left')
	col_order = VariableColumnDefn(title='#', valueGetter='order', width=40)
	col_resource = VariableColumnDefn(title='Resource', valueGetter='resource_name',
			width=150, align='left')
	col_values = VariableColumnDefn(title='Values', valueGetter=lambda x: str(x),
			isSpaceFilling=True, align='left')
	col_wait = VariableColumnDefn(title='Wait time', valueGetter='wait')
	col_const = VariableColumnDefn(checkStateGetter='use_const', title='Const. value',
			valueGetter='const')

	def __init__(self, parent, global_store, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		self.global_store = global_store

		# Panel.
		panel_box = wx.BoxSizer(wx.VERTICAL)

		## OLV.
		self.olv = ObjectListView.GroupListView(self)
		panel_box.Add(self.olv, proportion=1, flag=wx.ALL|wx.EXPAND)

		self.olv.SetColumns([self.col_name, self.col_order, self.col_resource, self.col_values,
				self.col_wait, self.col_const])
		self.olv.SetSortColumn(self.col_order)

		self.olv.cellEditMode = self.olv.CELLEDIT_DOUBLECLICK
		self.olv.Bind(ObjectListView.EVT_CELL_EDIT_STARTING, self.OnCellEditStarting)
		self.olv.Bind(ObjectListView.EVT_CELL_EDIT_FINISHING, self.OnCellEditFinishing)
		self.olv.Bind(ObjectListView.EVT_CELL_EDIT_FINISHED, self.OnCellEditFinished)

		## Buttons.
		button_box = wx.BoxSizer(wx.HORIZONTAL)
		panel_box.Add(button_box, proportion=0, flag=wx.ALL|wx.CENTER)

		### Row buttons.
		row_box = wx.BoxSizer(wx.HORIZONTAL)
		button_box.Add(row_box)

		add_button = wx.Button(self, wx.ID_ADD)
		add_button.Bind(wx.EVT_BUTTON, self.OnAddVariable)
		row_box.Add(add_button)

		remove_button = wx.Button(self, wx.ID_REMOVE)
		remove_button.Bind(wx.EVT_BUTTON, self.OnRemoveVariables)
		row_box.Add(remove_button)

		### Export buttons.
		export_box = wx.BoxSizer(wx.HORIZONTAL)
		button_box.Add(export_box, flag=wx.LEFT, border=20)

		save_button = wx.Button(self, wx.ID_SAVE, label='Save...')
		save_button.Bind(wx.EVT_BUTTON, self.OnSave)
		export_box.Add(save_button)

		load_button = wx.Button(self, wx.ID_OPEN, label='Load...')
		load_button.Bind(wx.EVT_BUTTON, self.OnLoad)
		export_box.Add(load_button)

		self.SetSizer(panel_box)

	def max_order(self):
		"""
		Find the highest-used order in the OLV.
		"""

		try:
			return max(x.order for x in self.olv.GetObjects())
		except ValueError:
			return 0

	def OnCellEditStarting(self, evt):
		col = evt.objectListView.columns[evt.subItemIndex]
		var = evt.rowModel

		# Ignore frivolous requests.
		if evt.rowIndex < 0:
			evt.Veto()
			return

		if col == self.col_values:
			def ok_callback(dlg):
				try:
					values = dlg.GetValue()
				except ValueError as e:
					MessageDialog(self, str(e), 'Invalid value').Show()
					return False

				(var.config, var.smooth_steps, var.smooth_from, var.smooth_to,
						var.smooth_transition, var.type, var.units) = values

				return True

			dlg = VariableEditor(self, ok_callback, title=var.name)
			dlg.SetValue(var.config, var.smooth_steps, var.smooth_from, var.smooth_to, var.smooth_transition,
					var.type, var.units)
			dlg.Show()

			# No need to use the default editor.
			evt.Veto()

	def OnCellEditFinishing(self, evt):
		col = evt.objectListView.columns[evt.subItemIndex]

		if col == self.col_name:
			var = evt.rowModel # With old name.
			var_new_name = evt.editor.Value

			if var_new_name == var.name:
				# Not actually changed.
				return

			# Attempt to add a new entry first.
			try:
				self.global_store.variables[var_new_name] = var
			except KeyError:
				MessageDialog(self, var_new_name, 'Variable name conflicts').Show()
				evt.Veto()
				return

			# Remove the old entry.
			del self.global_store.variables[var.name]

	def OnCellEditFinished(self, evt):
		col = evt.objectListView.columns[evt.subItemIndex]

		if col == self.col_order:
			self.olv.RebuildGroups()

	def OnSave(self, evt=None):
		"""
		Save all the rows in the OLV.
		"""

		try:
			save_pickled(self, self.olv.GetObjects(), extension='var', file_type='Variables')
		except IOError as e:
			MessageDialog(self, str(e), 'Save error').Show()
			return

	def OnLoad(self, evt=None):
		"""
		Load some rows to the OLV.
		"""

		try:
			values = load_pickled(self, extension='var', file_type='Variables')
		except IOError as e:
			MessageDialog(self, str(e), 'Load error').Show()
			return

		if values is not None:
			# Clear the OLV.
			for var in self.olv.GetObjects():
				del self.global_store.variables[var.name]
				self.olv.RemoveObject(var)

			conflicting_names = []
			for var in values:
				try:
					self.global_store.variables[var.name] = var
				except KeyError:
					conflicting_names.append(var.name)
					continue

				self.olv.AddObject(var)

			if conflicting_names:
				MessageDialog(self, ', '.join(conflicting_names), 'Variable names conflict').Show()

	def OnAddVariable(self, evt=None):
		"""
		Add a blank variable to the OLV.
		"""

		# Ensure that we get a unique name.
		with self.global_store.variables.lock:
			num = 1
			done = False
			while not done:
				name = 'New variable {0}'.format(num)
				var = OutputVariable(name=name, order=self.max_order()+1)

				try:
					self.global_store.variables[name] = var
				except KeyError:
					num += 1
				else:
					done = True

		self.olv.AddObject(var)

		# OLV likes to select a random item at this point.
		self.olv.DeselectAll()

	def OnRemoveVariables(self, evt=None):
		"""
		Remove all selected variables from the OLV.
		"""

		selected = self.olv.GetSelectedObjects()

		if selected:
			self.olv.RemoveObjects(selected)

		for row in selected:
			del self.global_store.variables[row.name]



import logging
log = logging.getLogger(__name__)

from threading import RLock
from time import time

from spacq.tool.box import Enum, Synchronized

"""
Hardware device abstraction interface.
"""


# PyVISA, Linux GPIB, PyVISA USB.
drivers = Enum(['pyvisa', 'lgpib', 'pyvisa_usb'])


# Try to import all available drivers.
available_drivers = []

try:
	import Gpib
	import gpib
except ImportError:
	pass
else:
	available_drivers.append(drivers.lgpib)

try:
	import visa
except ImportError:
	pass
else:
	available_drivers.append(drivers.pyvisa)
	available_drivers.append(drivers.pyvisa_usb)


class DeviceNotFoundError(Exception):
	"""
	Failure to connect to a device.
	"""

	pass


class DeviceTimeout(Exception):
	"""
	Timeout on action.
	"""

	pass


class IbstaBits(object):
	"""
	Status bits (in ibsta) as reported by Linux GPIB.
	"""

	DCAS = 0x1
	DTAS = 0x2
	LACS = 0x4
	TACS = 0x8
	ATN = 0x10
	CIC = 0x20
	REM = 0x40
	LOK = 0x80
	CMPL = 0x100
	EVENT = 0x200
	SPOLL = 0x400
	RQS = 0x800,
	SRQI = 0x1000
	END = 0x2000
	TIMO = 0x4000
	ERR = 0x8000


class SuperDevice(object):
	def _setup(self):
		"""
		Pre-connection setup.
		"""

		self.name = self.__class__.__name__

		self.resources = {}
		self.subdevices = {}

	def _connected(self):
		"""
		Post-connection setup.
		"""

		if hasattr(self, 'driver') and self.driver == drivers.lgpib:
			# Some devices don't assert the EOI line, so look for their EOS character instead.
			if hasattr(self, 'eos_char'):
				self.device.config(gpib.IbcEOSchar, ord(self.eos_char))
				self.device.config(gpib.IbcEOSrd, 1)

			# Gpib.Gpib doesn't complain if the device at the PAD doesn't actually exist.
			try:
				log.debug('GPIB device IDN: {0!r}'.format(self.idn))
			except gpib.GpibError as e:
				raise DeviceNotFoundError('Could not open device at "{0}".'.format(self.connection_resource), e)

		# Recursively.
		for name, subdev in self.subdevices.items():
			log.debug('Post-connection for subdevice "{0}".'.format(name))

			subdev._connected()


class AbstractDevice(SuperDevice):
	"""
	A class for controlling devices which can be connected to either via Ethernet and PyVISA or GPIB and Linux GPIB.
	"""

	max_timeout = 15 # s

	def _setup(self):
		self.multi_command = None
		self.responses_expected = 0

		SuperDevice._setup(self)

		self.lock = RLock()

		self.status = []

	def __init__(self, ip_address=None, gpib_board=0, gpib_pad=None, gpib_sad=0,
			usb_resource=None, autoconnect=True):
		"""
		Ethernet (tcpip::<ip_address>::instr):
			ip_address: Address on which the device is listening on port 111.

		GPIB (gpib[gpib_board]::<gpib_pad>[::<gpib_sad>]::instr):
			gpib_board: GPIB board index. Defaults to 0.
			gpib_pad: Primary address of the device.
			gpib_sad: Secondary address of the device. Defaults to 0.

		USB (usb_resource):
			usb_resource: VISA resource of the form: USB[board]::<vendor>::<product>::<serial>[::<interface>]::RAW

		autoconnect: Connect to the device upon instantiation.
		"""

		self._setup()

		log.info('Creating device "{0}".'.format(self.name))

		if ip_address is not None:
			if drivers.pyvisa in available_drivers:
				log.debug('Using PyVISA with ip_address="{0}".'.format(ip_address))
				self.driver = drivers.pyvisa
				self.connection_resource = {
					'resource_name': 'tcpip::{0}::instr'.format(ip_address),
				}
			else:
				raise NotImplementedError('PyVISA required, but not available.')
		elif gpib_pad is not None:
			if drivers.lgpib in available_drivers:
				log.debug('Using Linux GPIB with gpib_board="{0}", gpib_pad="{1}", '
						'gpib_sad="{2}".'.format(gpib_board, gpib_pad, gpib_sad))
				self.driver = drivers.lgpib
				self.connection_resource = {
					'name': gpib_board,
					'pad': gpib_pad,
					'sad': gpib_sad,
				}
			elif drivers.pyvisa in available_drivers:
				log.debug('Using PyVISA with gpib_board="{0}", gpib_pad="{1}", '
						'gpib_sad="{2}".'.format(gpib_board, gpib_pad, gpib_sad))
				self.driver = drivers.pyvisa
				self.connection_resource = {
					'resource_name': 'gpib{0}::{1}::{2}::instr'.format(gpib_board, gpib_pad, gpib_sad),
				}
			else:
				raise NotImplementedError('Linux GPIB or PyVISA required, but not available.')
		elif usb_resource is not None:
			if drivers.pyvisa_usb in available_drivers:
				log.debug('Using PyVISA with usb_resource="{0}"'.format(usb_resource))
				self.driver = drivers.pyvisa_usb
				self.connection_resource = {
					'resource_name': usb_resource,
				}
			else:
				raise NotImplementedError('PyVISA required, but not available.')
		else:
			raise ValueError('Either an IP, GPIB, or USB address must be specified.')

		if autoconnect:
			self.connect()

	def __repr__(self):
		return '<{0}>'.format(self.__class__.__name__)

	def connect(self):
		"""
		Make a connection to the device.
		"""

		log.info('Connecting to device "{0}" using {1} at "{2}".'.format(self.name, self.driver, self.connection_resource))

		if self.driver == drivers.pyvisa:
			try:
				self.device = visa.Instrument(**self.connection_resource)
			except visa.VisaIOError as e:
				raise DeviceNotFoundError('Could not open device at "{0}".'.format(self.connection_resource), e)
		elif self.driver == drivers.lgpib:
			try:
				self.device = Gpib.Gpib(**self.connection_resource)
			except gpib.GpibError as e:
				raise DeviceNotFoundError('Could not open device at "{0}".'.format(self.connection_resource), e)
		elif self.driver == drivers.pyvisa_usb:
			class USBDevice(visa.Instrument):
				"""
				Using USB devices with PyVISA requires a small hack: the object must be an Instrument, but we can't call Instrument.__init__.
				"""

				def __init__(self, *args, **kwargs):
					# Bypass the initialization in visa.Instrument, due to "send_end" not being valid for USB.
					visa.ResourceTemplate.__init__(self, *args, **kwargs)

			try:
				self.device = USBDevice(**self.connection_resource)
			except visa.VisaIOError as e:
				raise DeviceNotFoundError('Could not open device at "{0}".'.format(self.connection_resource), e)

		try:
			self._connected()
		except Exception as e:
			raise DeviceNotFoundError('Could not finish connection to device at "{0}".'.format(self.connection_resource), e)

	def multi_command_start(self):
		"""
		Redirect further commands to a buffer.
		"""

		log.debug('Starting multi-command message for device "{0}"'.format(self.name))

		if self.driver not in [drivers.pyvisa, drivers.lgpib]:
			raise NotImplementedError('Unsupported driver: "{0}".'.format(self.driver))

		self.multi_command = []
		self.responses_expected = 0

	@Synchronized()
	def multi_command_stop(self):
		"""
		Stop redirecting to a buffer, and send the buffered commands.

		Returns the results of queries if any were expected.
		"""

		log.debug('Stopping multi-command message for device "{0}"'.format(self.name))

		if self.multi_command is None:
			raise ValueError('Multi-command message not started.')
		elif not self.multi_command:
			# No commands.
			return []

		commands = self.multi_command
		# This ensures that write and ask will not buffer the real message.
		self.multi_command = None

		# Only commands not starting with "*" get a ":" prefix.
		commands = [cmd if cmd[0] == '*' else ':' + cmd for cmd in commands]
		message = ';'.join(commands)

		if self.responses_expected:
			result = self.ask(message)

			# FIXME: What if the response contains a meaningful ";" somewhere?
			return result.split(';', self.responses_expected - 1)
		else:
			self.write(message)

			return []

	@Synchronized()
	def write(self, message):
		"""
		Write to the device.

		Supports multi-command.
		"""

		if self.multi_command is not None:
			log.debug('Writing to multi-command buffer for device "{0}": {1!r}'.format(self.name, message))

			self.multi_command.append(message)
			return

		log.debug('Writing to device "{0}": {1!r}'.format(self.name, message))

		if self.driver == drivers.pyvisa:
			try:
				self.device.write(message)
			except visa.VisaIOError as e:
				if e.error_code == visa.VI_ERROR_TMO:
					raise DeviceTimeout(e)
				else:
					raise
		elif self.driver == drivers.lgpib:
			try:
				self.device.write(message)
			except gpib.GpibError as e:
				if 'timeout' in e.message:
					raise DeviceTimeout(e)
				else:
					raise
		elif self.driver == drivers.pyvisa_usb:
			# Send the message raw.
			visa.vpp43.write(self.device.vi, message)

	@Synchronized()
	def read_raw(self, chunk_size=512):
		"""
		Read everything the device has to say and return it exactly.
		"""

		log.debug('Reading from device "{0}".'.format(self.name))

		buf = ''

		if self.driver in [drivers.pyvisa, drivers.pyvisa_usb]:
			try:
				buf = self.device.read_raw()
			except visa.VisaIOError as e:
				if e.error_code == visa.VI_ERROR_TMO:
					raise DeviceTimeout(e)
				else:
					raise
		elif self.driver == drivers.lgpib:
			status = 0
			while status == 0:
				try:
					buf += self.device.read(len=chunk_size)
				except gpib.GpibError as e:
					if 'timeout' in e.message:
						raise DeviceTimeout(e)
					else:
						raise

				status = self.device.ibsta() & IbstaBits.END

		log.debug('Read from device "{0}": {1!r}'.format(self.name, buf))

		return buf

	def read(self):
		"""
		Read from the device, but strip terminating whitespace.
		"""

		return self.read_raw().rstrip()

	@Synchronized()
	def ask_raw(self, message):
		"""
		Write, then read_raw.
		"""

		self.write(message)
		return self.read_raw()

	@Synchronized()
	def ask(self, message):
		"""
		Write, then read.

		Supports multi-command.
		"""

		self.write(message)

		if self.multi_command is None:
			return self.read()
		else:
			self.responses_expected += 1

	def close(self):
		"""
		Close the connection, if possible.
		"""

		log.debug('Closing device: {0}'.format(self.name))

		if self.driver in [drivers.pyvisa, drivers.pyvisa_usb]:
			self.device.close()

	def find_resource(self, path):
		"""
		Return a Resource given a resource path spec.

		eg. ('subdevice A', 'subdevice B', 'resource C') -> Resource
		"""

		log.debug('Looking for resource {0}.'.format(path))

		if len(path) < 1:
			raise ValueError('No path provided.')

		# Keep track of the last device in the tree so far.
		dev = self
		# Keep track of the path so far.
		traversed = ()

		while len(path) > 1:
			try:
				dev = dev.subdevices[path[0]]
			except KeyError:
				raise ValueError('No subdevice "{0}" in {1}.'.format(path[0], traversed))

			traversed += path[:1]
			path = path[1:]

		try:
			return dev.resources[path[0]]
		except KeyError:
			raise ValueError('No resource "{0}" in {1}.'.format(path[0], traversed))

	@property
	def idn(self):
		"""
		Ask the device for identification.
		"""

		if self.driver in [drivers.pyvisa, drivers.lgpib]:
			return self.ask('*idn?')

	@property
	def opc(self):
		"""
		Wait until the device is done.
		"""

		end_time = time() + self.max_timeout

		while True:
			if self.driver in [drivers.pyvisa, drivers.lgpib]:
				try:
					self.ask('*opc?')
				except DeviceTimeout:
					if time() > end_time:
						raise
				else:
					break


class AbstractSubdevice(SuperDevice):
	"""
	A subdevice (eg. channel) of a hardware device.
	"""

	def _setup(self):
		SuperDevice._setup(self)

		# Synchronized methods should use the device lock.
		self.lock = self.device.lock if self.device else None

	def __init__(self, device):
		self.device = device

		self._setup()



import wx
from wx.lib.agw.floatspin import FloatSpin

from spacq.interface.units import Quantity

from ..tool.box import Dialog

"""
Resource scaling configuration.
"""


class ScalingSettings(object):
	"""
	Set up scaling of the form:
		f(x) = a * x * (10 ** b) + c
	"""

	def __init__(self):
		self.linear_scale = 1
		self.exponential_scale = 0
		self.offset = 0

	def transform(self, x):
		"""
		Perform a transform according to the scaling.
		"""

		if self.offset == 0:
			return self.linear_scale * x * (10 ** self.exponential_scale)
		elif isinstance(x, Quantity):
			return self.linear_scale * x * (10 ** self.exponential_scale) + Quantity(self.offset, x.original_units)
		else:
			return self.linear_scale * x * (10 ** self.exponential_scale) + self.offset


class ScalingSettingsDialog(Dialog):
	def __init__(self, parent, ok_callback, *args, **kwargs):
		Dialog.__init__(self, parent, title='Scaling settings')

		self.ok_callback = ok_callback

		# Dialog.
		dialog_box = wx.BoxSizer(wx.VERTICAL)

		## Settings.
		settings_box = wx.FlexGridSizer(rows=3, cols=2, hgap=5)
		dialog_box.Add(settings_box, flag=wx.EXPAND|wx.ALL, border=5)

		### Linear scale.
		settings_box.Add(wx.StaticText(self, label='Linear scale:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.linear_scale_input = FloatSpin(self, value=0, min_val=-1e9, max_val=1e9,
				increment=1, digits=5)
		settings_box.Add(self.linear_scale_input, flag=wx.EXPAND)

		### Exponential scale.
		settings_box.Add(wx.StaticText(self, label='Exponential scale:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.exponential_scale_input = FloatSpin(self, value=0, min_val=-100, max_val=100,
				increment=1, digits=2)
		settings_box.Add(self.exponential_scale_input, flag=wx.EXPAND)

		### Offset.
		settings_box.Add(wx.StaticText(self, label='Offset:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.offset_input = FloatSpin(self, value=0, min_val=-1e9, max_val=1e9,
				increment=1, digits=5, size=(200, -1))
		settings_box.Add(self.offset_input, flag=wx.EXPAND)

		## End buttons.
		button_box = wx.BoxSizer(wx.HORIZONTAL)
		dialog_box.Add(button_box, flag=wx.CENTER|wx.ALL, border=5)

		ok_button = wx.Button(self, wx.ID_OK)
		self.Bind(wx.EVT_BUTTON, self.OnOk, ok_button)
		button_box.Add(ok_button)

		cancel_button = wx.Button(self, wx.ID_CANCEL)
		button_box.Add(cancel_button)

		self.SetSizerAndFit(dialog_box)

	def GetValue(self):
		result = ScalingSettings()

		result.linear_scale = self.linear_scale_input.GetValue()
		result.exponential_scale = self.exponential_scale_input.GetValue()
		result.offset = self.offset_input.GetValue()

		return result

	def SetValue(self, value):
		self.linear_scale_input.SetValue(value.linear_scale)
		self.exponential_scale_input.SetValue(value.exponential_scale)
		self.offset_input.SetValue(value.offset)

	def OnOk(self, evt=None):
		self.ok_callback(self)

		self.Destroy()



from unittest import main

from ... import abc1234
from .. import mock_abc1234

from ...tests.server.test_abc1234 import ABC1234Test


# Don't lose the real device.
real_ABC1234 = abc1234.ABC1234
is_mock = ABC1234Test.mock


def setup():
	# Run the tests with a fake device.
	abc1234.ABC1234 = mock_abc1234.MockABC1234
	ABC1234Test.mock = True

def teardown():
	# Restore the real device for any remaining tests.
	abc1234.ABC1234 = real_ABC1234
	ABC1234Test.mock = is_mock


if __name__ == '__main__':
	main()



import logging
log = logging.getLogger(__name__)

from functools import partial, wraps
from itertools import izip, repeat
from threading import Condition, Thread
from time import sleep, time

from spacq.tool.box import flatten


def update_current_f(f):
	@wraps(f)
	def wrapped(self):
		self.current_f = f.__name__

		log.debug('Entering function: {0}'.format(self.current_f))

		return f(self)

	return wrapped


class PulseConfiguration(object):
	"""
	The configuration necessary to execute a pulse program with a device.
	"""

	# All the directly-used attributes.
	awg_attrs = ['channels', 'clear_channels', 'enabled', 'run_mode', 'sampling_rate', 'trigger']
	oscilloscope_attrs = ['acquiring', 'fastframe', 'fastframe_count', 'fastframe_sum', 'stopafter']

	@staticmethod
	def verify_device(name, device, attributes):
		if device is None:
			raise TypeError('No "{0}" device configured'.format(name))

		d = dir(device)

		for attribute in attributes:
			if attribute not in d:
				raise TypeError('Given "{0}" device lacks "{1}"'.format(name, attribute))

	def __init__(self, program, channels, awg, oscilloscope):
		self.verify_device('AWG', awg, self.awg_attrs)
		self.verify_device('Oscilloscope', oscilloscope, self.oscilloscope_attrs)

		self.program = program
		self.channels = channels
		self.awg = awg
		self.oscilloscope = oscilloscope


class SweepController(object):
	"""
	A simple controller for a sweep of several variables.

	init -> next -> transition -> write -> dwell -> pulse -> read -> ramp_down -> end
	^       ^                                  |_____________^  |            |
	|       |___________________________________________________|            |
	|________________________________________________________________________|
	"""

	def __init__(self, resources, variables, num_items, measurement_resources, measurement_variables,
			pulse_config=None, continuous=False):
		self.resources = resources
		self.variables = variables
		self.num_items = num_items
		self.measurement_resources = measurement_resources
		self.measurement_variables = measurement_variables
		self.pulse_config = pulse_config
		self.continuous = continuous

		# The callbacks should be set before calling run(), if necessary.
		self.data_callback, self.close_callback, self.write_callback, self.read_callback = [None] * 4
		self.general_exception_handler = None
		self.resource_exception_handler = None

		self.devices_configured = False

		self.current_f = None

		self.item = -1

		self.paused = False
		self.pause_lock = Condition()

		self.last_continuous = False
		self.done = False
		self.aborting = False
		self.abort_fatal = False

		self.sweep_start_time = time()
		self.first_time_point = None

	def create_iterator(self, pos):
		"""
		Create an iterator for an order of variables.
		"""

		return izip(*(iter(var) for var in self.variables[pos]))

	def ramp(self, resources, values_from, values_to, steps):
		"""
		Slowly sweep the resources.
		"""

		thrs = []
		for (name, resource), value_from, value_to, resource_steps in zip(resources,
				values_from, values_to, steps):
			if resource is None:
				continue

			kwargs = {}
			if self.resource_exception_handler is not None:
				kwargs['exception_callback'] = partial(self.resource_exception_handler, name, write=True)

			thr = Thread(target=resource.sweep, args=(value_from, value_to, resource_steps), kwargs=kwargs)
			thrs.append(thr)
			thr.daemon = True
			thr.start()

		for thr in thrs:
			thr.join()

	def write_resource(self, name, resource, value):
		"""
		Write a value to a resource and handle exceptions.
		"""

		try:
			resource.value = value
		except Exception as e:
			if self.resource_exception_handler is not None:
				self.resource_exception_handler(name, e, write=True)
			return

	def read_resource(self, name, resource, save_callback):
		"""
		Read a value from a resource and handle exceptions.
		"""

		try:
			value = resource.value
		except Exception as e:
			if self.resource_exception_handler is not None:
				self.resource_exception_handler(name, e, write=False)
			return

		save_callback(value)

	def run(self, next_f=None):
		"""
		Run the sweep.
		"""

		try:
			if next_f is None:
				next_f = self.init

			# Trampoline.
			while next_f is not None:
				f_name = next_f.__name__

				if self.paused:
					log.debug('Paused before function: {0}'.format(f_name))

					with self.pause_lock:
						self.pause_lock.wait()

				if self.aborting:
					log.debug('Aborting before function: {0}'.format(f_name))

					if not self.abort_fatal:
						self.continuous = False
						self.ramp_down()

					return

				log.debug('Starting function: {0}'.format(f_name))

				try:
					next_f = next_f()
				except Exception as e:
					if self.general_exception_handler is not None:
						self.general_exception_handler(f_name, e)
					else:
						log.exception('Caught exception in function: {0}'.format(f_name))

					# Attempt to exit normally at this point.
					next_f = None
		finally:
			self.end()

	@update_current_f
	def init(self):
		"""
		Initialize values and possibly devices.
		"""

		self.iterators = None
		self.current_values = None
		self.last_values = None

		self.item = -1

		if not self.devices_configured:
			log.debug('Configuring devices')

			if self.pulse_config is not None:
				# AWG
				awg = self.pulse_config.awg

				awg.sampling_rate = self.pulse_config.program.frequency
				awg.run_mode = 'triggered'

			self.devices_configured = True

		return self.next

	@update_current_f
	def next(self):
		"""
		Get the next set of values from the iterators.
		"""

		self.item += 1
		if self.current_values is not None:
			self.last_values = self.current_values[:]

		if self.iterators is None:
			# First time around.
			self.iterators = []
			for pos in xrange(len(self.variables)):
				self.iterators.append(self.create_iterator(pos))

			self.current_values = [it.next() for it in self.iterators]
			self.changed_indices = range(len(self.variables))
		else:
			pos = len(self.variables) - 1
			while pos >= 0:
				try:
					self.current_values[pos] = self.iterators[pos].next()
					break
				except StopIteration:
					self.iterators[pos] = self.create_iterator(pos)
					self.current_values[pos] = self.iterators[pos].next()

					pos -= 1

			self.changed_indices = range(pos, len(self.variables))

		return self.transition

	@update_current_f
	def transition(self):
		"""
		Perform a transition for variables, as required.
		"""

		if self.last_values is None:
			# Smooth set from const.
			steps, resources, from_values, to_values = [], [], [], []

			for pos in xrange(len(self.variables)):
				# Extract values for this group.
				group_vars, group_resources, current_values = (self.variables[pos],
						self.resources[pos], self.current_values[pos])

				for var, resource, current_value in zip(group_vars, group_resources,
						current_values):
					if var.use_const or not var.smooth_from:
						continue

					steps.append(var.smooth_steps)
					resources.append(resource)
					from_values.append(var.with_type(var.const))
					to_values.append(current_value)

			self.ramp(resources, from_values, to_values, steps)
		else:
			# The first changed group is simply stepping; all others rolled over.
			affected_groups = self.changed_indices[1:]

			steps, resources, from_values, to_values = [], [], [], []

			for pos in affected_groups:
				# Extract values for this group.
				group_vars, group_resources, current_values, last_values = (self.variables[pos],
						self.resources[pos], self.current_values[pos], self.last_values[pos])

				for var, resource, current_value, last_value in zip(group_vars, group_resources,
						current_values, last_values):
					if var.use_const or not var.smooth_transition:
						continue

					steps.append(var.smooth_steps)
					resources.append(resource)
					from_values.append(last_value)
					to_values.append(current_value)

			self.ramp(resources, from_values, to_values, steps)

		return self.write

	@update_current_f
	def write(self):
		"""
		Write the next values to their resources.
		"""

		thrs = []
		for pos in self.changed_indices:
			for i, ((name, resource), value) in enumerate(zip(self.resources[pos], self.current_values[pos])):
				if resource is not None:
					thr = Thread(target=self.write_resource, args=(name, resource, value))
					thrs.append(thr)
					thr.daemon = True
					thr.start()

				if self.write_callback is not None:
					self.write_callback(pos, i, value)

		for thr in thrs:
			thr.join()

		return self.dwell

	@update_current_f
	def dwell(self):
		"""
		Wait for all changed variables.
		"""

		delay = max(var._wait.value for pos in self.changed_indices for var in self.variables[pos])
		sleep(delay)

		if self.pulse_config is not None:
			return self.pulse
		else:
			return self.read

	@update_current_f
	def pulse(self):
		"""
		Run through the pulse program.
		"""

		if self.pulse_config.channels:
			waveforms = self.pulse_config.program.generate_waveforms()
			times = self.pulse_config.program.times_average

			# AWG
			awg = self.pulse_config.awg
			awg.enabled = False

			awg.clear_channels()

			channels = []
			for output, number in self.pulse_config.channels.items():
				channel = awg.channels[number]

				waveform, markers = waveforms[output]
				channel.set_waveform(waveform, markers, name=output)

				channels.append(channel)

			for channel in channels:
				channel.enabled = True

			awg.enabled = True

			# Oscilloscope
			osc = self.pulse_config.oscilloscope
			osc.acquiring = False

			if times > 1:
				osc.fastframe = True
				osc.fastframe_sum = 'average'
				osc.fastframe_count = times + 1

				if osc.fastframe_count != times + 1:
					raise ValueError('Cannot average {0} times; check the oscilloscope'.format(times))
			else:
				osc.fastframe = False

			osc.stopafter = 'sequence'

			awg.opc
			osc.opc

			osc.acquiring = True
			# Wait for the oscilloscope to ready the trigger.
			sleep(1)

			# All together now!
			trigger = awg.trigger
			delay = self.pulse_config.program.acq_delay.value

			for _ in repeat(None, times):
				trigger()
				awg.opc

				end_time = time() + delay
				time_diff = end_time - time()
				while time_diff > 0:
					sleep(time_diff)
					time_diff = end_time - time()

			osc.opc

			acqs = osc.acquisitions
			if acqs != times:
				raise ValueError('Incorrect number of acquisitions made: {0}'.format(acqs))

		return self.read

	@update_current_f
	def read(self):
		"""
		Take measurements.
		"""

		measurements = [None] * len(self.measurement_resources)

		thrs = []
		for i, (name, resource) in enumerate(self.measurement_resources):
			if resource is not None:
				def save_callback(value, i=i):
					measurements[i] = value
					if self.read_callback is not None:
						self.read_callback(i, value)

				thr = Thread(target=self.read_resource, args=(name, resource, save_callback))
				thrs.append(thr)
				thr.daemon = True
				thr.start()

		for thr in thrs:
			thr.join()

		if self.data_callback is not None:
			if self.first_time_point is None:
				cur_time = 0
				self.first_time_point = time()
			else:
				cur_time = time() - self.first_time_point

			self.data_callback(cur_time, tuple(flatten(self.current_values)), tuple(measurements))

		if self.item == self.num_items - 1:
			self.item += 1

			return self.ramp_down
		else:
			return self.next

	@update_current_f
	def ramp_down(self):
		"""
		Sweep from the last values to const.
		"""

		if not self.current_values:
			return

		# Smooth set to const.
		steps, resources, from_values, to_values = [], [], [], []

		for pos in xrange(len(self.variables)):
			# Extract values for this group.
			group_vars, group_resources, current_values = (self.variables[pos],
					self.resources[pos], self.current_values[pos])

			for var, resource, current_value in zip(group_vars, group_resources,
					current_values):
				if var.use_const or not var.smooth_to:
					continue

				steps.append(var.smooth_steps)
				resources.append(resource)
				from_values.append(current_value)
				to_values.append(var.with_type(var.const))

		self.ramp(resources, from_values, to_values, steps)

		if self.continuous and not self.last_continuous:
			return self.init

	@update_current_f
	def end(self):
		"""
		The sweep is over.
		"""

		assert not self.done
		self.done = True

		if self.close_callback is not None:
			self.close_callback()

	def pause(self):
		log.debug('Pausing.')

		self.paused = True

		log.debug('Paused.')

	def unpause(self):
		log.debug('Unpausing.')

		with self.pause_lock:
			self.paused = False
			self.pause_lock.notify()

		log.debug('Unpaused.')

	def abort(self, fatal=False):
		"""
		Ending abruptly for any reason.
		"""

		log.debug('Aborting.')

		self.aborting = True
		self.abort_fatal = fatal

		if self.abort_fatal:
			log.warning('Aborting fatally.')

		self.unpause()



import logging
log = logging.getLogger(__name__)


name = 'IQC'

from . import voltage_source
models = [voltage_source]
log.debug('Found models for "{0}": {1}'.format(name, ''.join(str(x) for x in models)))

from .mock import mock_voltage_source
mock_models = [mock_voltage_source]
log.debug('Found mock models for "{0}": {1}'.format(name, ''.join(str(x) for x in mock_models)))



from copy import copy, deepcopy
from os.path import basename, dirname

from ..units import Quantity
from .parser import Parser, PulseError, PulseSyntaxError
from .tree import Environment

"""
Pulse programs as compiled entities.
"""


class Program(object):
	"""
	A prepared, compiled pulse program.
	"""

	filename = ''

	@classmethod
	def from_string(cls, s):
		"""
		Create a program from a string.
		"""

		env = Environment()
		ast = Parser()(s)

		return Program(env, ast)

	@classmethod
	def from_file(cls, path):
		"""
		Load a program from a file.
		"""

		with open(path) as f:
			prog_lines = f.readlines()

		p = cls.from_string(''.join(prog_lines))
		p._env.cwd = dirname(path)
		p.filename = basename(path)

		return p

	def __init__(self, env, ast):
		"""
		Given a blank Environment and AST, prepare a program.
		"""

		self._env = env
		self._ast = ast

		for stage in self._env.prep_stages:
			self._env.stage = stage
			self._env.traverse_tree(self._ast)

			if self._env.errors:
				raise PulseSyntaxError(self._env.format_errors())

		# Output channel numbers.
		self.output_channels = dict((name, None) for name, type in self._env.variables.items() if type == 'output')

		# Output device labels.
		self.awg = ''
		self.oscilloscope = ''

		# Same structure as values, but optional string-only resource labels and matching Resource objects.
		self.resource_labels = {}
		self.resources = {}

		# Program averaging.
		self.times_average = 1
		self.acq_delay = Quantity(0, 's')

	@property
	def all_values(self):
		return self._env.all_values

	@property
	def frequency(self):
		return self._env.frequency

	@frequency.setter
	def frequency(self, value):
		self._env.frequency = value

	@property
	def missing_values(self):
		return self._env.missing_values

	@property
	def values(self):
		return self._env.values

	@property
	def variables(self):
		return self._env.variables

	def set_value(self, parameter, value):
		"""
		Set a value, even if it has already been set previously.
		"""

		self._env.values[parameter] = value

	def generate_waveforms(self, dry_run=False):
		"""
		Generate the waveforms, given that the values are all filled in.
		"""

		self._env.stage = self._env.stages.waveforms
		self._env.dry_run = dry_run
		self._env.missing_shapes = set()
		self._env.errors = []
		self._env.traverse_tree(self._ast)

		if self._env.errors:
			raise PulseError(self._env.format_errors())

		return self._env.waveforms

	@property
	def with_resources(self):
		"""
		Produce a copy object, with a cloned Environment, and with all resources which exist mutated to point to the corresponding values.

		Note: Because the Resource objects must be shared between copies, there may only ever exist one copy at a time.
		"""

		result = copy(self)

		# We plan to modify the values in the Environment.
		result._env = deepcopy(self._env)

		for parameter, label in self.resource_labels.items():
			def setter(x, parameter=parameter):
				result._env.values[parameter] = x

			res = result.resources[parameter]
			res.setter = setter

			# Type checking.
			var_type = result._env.variables[parameter[0]]
			if var_type == 'delay':
				res.units = 's'
			elif var_type == 'int':
				pass
			elif var_type == 'pulse':
				if parameter[1] == 'amplitude':
					res.units = 'V'
				elif parameter[1] == 'length':
					res.units = 's'

		return result



from unittest import main

from ... import dpo7104
from .. import mock_dpo7104

from ...tests.server.test_dpo7104 import DPO7104Test


# Don't lose the real device.
real_DPO7104 = dpo7104.DPO7104
is_mock = DPO7104Test.mock


def setup():
	# Run the tests with a fake device.
	dpo7104.DPO7104 = mock_dpo7104.MockDPO7104
	DPO7104Test.mock = True

def teardown():
	# Restore the real device for any remaining tests.
	dpo7104.DPO7104 = real_DPO7104
	DPO7104Test.mock = is_mock


if __name__ == '__main__':
	main()



from matplotlib import pyplot
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from mpl_toolkits.mplot3d import axes3d
import numpy
import wx

"""
An embeddable three-dimensional surface plot.
"""


class SurfacePlot(object):
	"""
	A surface plot.
	"""

	alpha = 0.8

	def __init__(self, parent, style='surface'):
		self.style = style

		self.figure = pyplot.figure()
		self.canvas = FigureCanvas(parent, wx.ID_ANY, self.figure)

		self.axes = axes3d.Axes3D(self.figure)
		self.surface = None

	def __del__(self):
		try:
			self.close()
		except Exception:
			pass

	@property
	def control(self):
		"""
		A drawable control.
		"""

		return self.canvas

	def close(self):
		"""
		Inform pyplot that this figure is no longer required.
		"""

		pyplot.close(self.figure.number)

	def set_surface_data(self, data):
		"""
		Set the surface data based on the data tuple.
		"""

		if self.surface is not None:
			self.axes.collections.remove(self.surface)
			self.surface = None

		if data is None:
			return

		surface_data, x_bounds, y_bounds = data

		# Number of values along each axis.
		y_num, x_num = surface_data.shape
		# The equally-spaced values along each axis.
		x_values = numpy.linspace(*x_bounds, num=x_num)
		y_values = numpy.linspace(*y_bounds, num=y_num)
		# The meshgrid of values.
		x, y = numpy.meshgrid(x_values, y_values)

		if self.style == 'surface':
			# Just a regular surface.
			self.surface = self.axes.plot_surface(x, y, surface_data, alpha=self.alpha)
		elif self.style == 'waveform':
			# Waveform style shows individual waveforms nicely.
			self.surface = self.axes.plot_wireframe(x, y, surface_data, cstride=100000)

	surface_data = property(fset=set_surface_data)

	@property
	def x_label(self):
		"""
		The x axis label.
		"""
		return self.axes.get_xlabel()

	@x_label.setter
	def x_label(self, value):
		self.axes.set_xlabel(value)

	@property
	def y_label(self):
		"""
		The y axis label.
		"""
		return self.axes.get_ylabel()

	@y_label.setter
	def y_label(self, value):
		self.axes.set_ylabel(value)

	@property
	def z_label(self):
		"""
		The z axis label.
		"""
		return self.axes.get_zlabel()

	@z_label.setter
	def z_label(self, value):
		self.axes.set_zlabel(value)

	def redraw(self):
		self.canvas.draw()



from numpy import array, compress, zeros
import wx
from wx.lib.mixins.listctrl import ListCtrlAutoWidthMixin

from spacq.interface.list_columns import ListParser


"""
Embeddable, generic, virtual, tabular display.
"""


class VirtualListCtrl(wx.ListCtrl, ListCtrlAutoWidthMixin):
	"""
	A generic virtual list.
	"""

	max_value_len = 250 # Characters.

	@staticmethod
	def find_type(value):
		"""
		Determine the type of a column based on a single value.

		The type is one of: scalar, list, string.
		"""

		try:
			float(value)
		except ValueError:
			pass
		else:
			return 'scalar'

		try:
			ListParser()(value)
		except ValueError:
			pass
		else:
			return 'list'

		return 'string'

	def __init__(self, parent, *args, **kwargs):
		wx.ListCtrl.__init__(self, parent,
				style=wx.LC_REPORT|wx.LC_VIRTUAL|wx.LC_HRULES|wx.LC_VRULES,
				*args, **kwargs)

		ListCtrlAutoWidthMixin.__init__(self)

		self.reset()

	def reset(self):
		self.headings = []
		self.data = array([])
		self.filtered_data = None
		self.display_data = array([])

		self.types = []

	def refresh_with_values(self, data):
		self.ItemCount = len(data)

		if self.ItemCount > 0:
			self.display_data = zeros(data.shape, dtype='|S{0}'.format(self.max_value_len))

			for i, _ in enumerate(self.headings):
				# Truncate for display.
				self.display_data[:,i] = [x[:self.max_value_len] for x in data[:,i]]

		self.Refresh()

	def apply_filter(self, f, afresh=False):
		"""
		Set the data to be the old data, along with the application of a filter.

		f is a function of two parameters: the index of the row and the row itself.
		f must return True if the row is to be kept and False otherwise.

		If afresh is True, all old filtered data is discarded.
		Otherwise, a new filter can be quickly applied.
		"""

		if afresh:
			self.filtered_data = None

		if self.filtered_data is not None:
			original_set = self.filtered_data
		else:
			original_set = self.data

		self.filtered_data = compress([f(i, x) for i, x in enumerate(original_set)], original_set, axis=0)

		self.refresh_with_values(self.filtered_data)

	def GetValue(self, types=None):
		# Get all types by default.
		if types is None:
			types = set(self.types)
		else:
			types = set(types)

		# Find column indices of the correct type.
		idxs = [i for i, t in enumerate(self.types) if t in types]

		if self.filtered_data is not None:
			data = self.filtered_data
		else:
			data = self.data

		return ([self.headings[i] for i in idxs], data[:,idxs], [self.types[i] for i in idxs])

	def SetValue(self, headings, data):
		"""
		headings: A list of strings.
		data: A 2D NumPy array.
		"""

		self.ClearAll()
		self.reset()

		self.headings = headings
		self.data = data

		self.refresh_with_values(self.data)

		if self.ItemCount > 0:
			width, height = self.GetSize()
			# Give some room for the scrollbar.
			col_width = (width - 50) / len(self.headings)

			for i, heading in enumerate(self.headings):
				self.InsertColumn(i, heading, width=col_width)

				type = self.find_type(data[0,i])
				self.types.append(type)

	def OnGetItemText(self, item, col):
		"""
		Return cell value for LC_VIRTUAL.
		"""

		return self.display_data[item,col]


class TabularDisplayPanel(wx.Panel):
	"""
	A panel to display arbitrary tabular data.
	"""

	def __init__(self, parent, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		# Panel.
		panel_box = wx.BoxSizer(wx.VERTICAL)

		## Table.
		self.table = VirtualListCtrl(self)
		panel_box.Add(self.table, proportion=1, flag=wx.EXPAND)

		self.SetSizer(panel_box)

	def __len__(self):
		return self.table.ItemCount

	def from_csv_data(self, has_header, values):
		"""
		Import the given CSV data into the table.

		If has_header is True, the first row is treated specially.
		"""

		if has_header:
			headers, rows = values[0], array(values[1:])
		else:
			headers, rows = [''] * len(values[0]), array(values)

		# Ensure that all columns have a header.
		for i, header in enumerate(headers):
			if not header:
				headers[i] = 'Column {0}'.format(i + 1)

		self.SetValue(headers, rows)

	def GetValue(self, *args, **kwargs):
		return self.table.GetValue(*args, **kwargs)

	def SetValue(self, headings, values):
		self.table.SetValue(headings, values)


class TabularDisplayFrame(wx.Frame):
	def __init__(self, parent, *args, **kwargs):
		wx.Frame.__init__(self, parent, *args, **kwargs)

		# Frame.
		frame_box = wx.BoxSizer(wx.VERTICAL)

		## Display panel.
		self.display_panel = TabularDisplayPanel(self)
		frame_box.Add(self.display_panel, proportion=1, flag=wx.EXPAND)

		self.SetSizer(frame_box)



from pyparsing import (delimitedList, ParseBaseException, Regex, Suppress)
from re import IGNORECASE

"""
Tools for parsing list columns on import.
"""


def ListParser():
	"""
	A parser for list columns, where each list is composed of pairs of values.
	"""

	value = Regex(r'[-+]?[0-9]+(?:\.[0-9]*)?(?:e[-+]?[0-9]+)?', IGNORECASE)
	value.setParseAction(lambda toks: float(toks[0]))

	item = Suppress('(') + value + Suppress(',') + value + Suppress(')')
	item.setParseAction(tuple)

	lst = Suppress('[') + delimitedList(item) + Suppress(']')
	lst.setParseAction(list)

	def parse(s):
		try:
			return lst.parseString(s).asList()
		except ParseBaseException as e:
			raise ValueError(e)

	return parse



import wx

from .....tool.box import Dialog

"""
Plot configuration.
"""


class AxisSelectionPanel(wx.Panel):
	"""
	A panel for choosing the headings to be used for the axes.
	"""

	def __init__(self, parent, axes, headings, selection_callback, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		self.selection_callback = selection_callback

		# Panel.
		panel_box = wx.BoxSizer(wx.HORIZONTAL)

		## Axes.
		self.axis_lists = []

		for axis in axes:
			axis_static_box = wx.StaticBox(self, label=axis)
			axis_box = wx.StaticBoxSizer(axis_static_box, wx.VERTICAL)
			panel_box.Add(axis_box, proportion=1, flag=wx.EXPAND|wx.ALL, border=5)

			axis_list = wx.ListBox(self, choices=headings)
			axis_list.SetMinSize((-1, 300))
			self.Bind(wx.EVT_LISTBOX, self.OnAxisSelection, axis_list)
			axis_box.Add(axis_list, proportion=1, flag=wx.EXPAND)

			self.axis_lists.append(axis_list)

		self.SetSizer(panel_box)

	def OnAxisSelection(self, evt=None):
		"""
		Announce the latest selection.
		"""

		result = [None if list.Selection == wx.NOT_FOUND else list.Selection for
				list in self.axis_lists]

		self.selection_callback(result)


class PlotSetupDialog(Dialog):
	"""
	Plot configuration dialog.
	"""

	def __init__(self, parent, headings, axis_names, *args, **kwargs):
		Dialog.__init__(self, parent, *args, **kwargs)

		self.axes = [None for _ in axis_names]

		# Dialog.
		dialog_box = wx.BoxSizer(wx.VERTICAL)

		## Axis setup.
		axis_panel = AxisSelectionPanel(self, axis_names, headings, self.OnAxisSelection)
		dialog_box.Add(axis_panel)

		## End buttons.
		button_box = wx.BoxSizer(wx.HORIZONTAL)
		dialog_box.Add(button_box, flag=wx.CENTER)

		self.ok_button = wx.Button(self, wx.ID_OK)
		self.ok_button.Disable()
		self.Bind(wx.EVT_BUTTON, self.OnOk, self.ok_button)
		button_box.Add(self.ok_button)

		cancel_button = wx.Button(self, wx.ID_CANCEL)
		button_box.Add(cancel_button)

		self.SetSizerAndFit(dialog_box)

	def OnAxisSelection(self, values):
		self.axes = values

		self.ok_button.Enable(all(axis is not None for axis in self.axes))

	def OnOk(self, evt=None):
		if self.make_plot():
			self.Destroy()

			return True



from functools import partial
from threading import Thread
import wx

from spacq.iteration.variables import OutputVariable
from spacq.tool.box import sift

from ..tool.box import MessageDialog


class SmoothResetPanel(wx.Panel):
	"""
	A panel to change variables smoothly to and from preset values.
	"""

	def __init__(self, parent, global_store, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		self.global_store = global_store

		# Panel.
		panel_box = wx.BoxSizer(wx.VERTICAL)

		## Reset.
		reset_static_box = wx.StaticBox(self, label='Smooth reset')
		reset_box = wx.StaticBoxSizer(reset_static_box, wx.VERTICAL)
		panel_box.Add(reset_box, flag=wx.CENTER|wx.ALL, border=10)

		### To zero.
		self.to_button = wx.Button(self, label='To zero')
		self.Bind(wx.EVT_BUTTON, self.OnResetToZero, self.to_button)
		reset_box.Add(self.to_button, flag=wx.EXPAND)

		### From zero.
		self.from_button = wx.Button(self, label='From zero')
		self.Bind(wx.EVT_BUTTON, self.OnResetFromZero, self.from_button)
		reset_box.Add(self.from_button, flag=wx.EXPAND)

		### Steps.
		steps_static_box = wx.StaticBox(self, label='Steps')
		steps_box = wx.StaticBoxSizer(steps_static_box, wx.VERTICAL)
		reset_box.Add(steps_box, flag=wx.EXPAND)

		self.reset_steps_input = wx.SpinCtrl(self, min=1, initial=10)
		steps_box.Add(self.reset_steps_input)

		self.SetSizer(panel_box)

	def choose_variables(self):
		"""
		Return all the selected variables, ensuring that their resources are valid.
		"""

		all_vars = sift(self.global_store.variables.values(), OutputVariable)
		vars = [var for var in all_vars if var.enabled and var.use_const and var.resource_name]

		missing_resources = []
		unwritable_resources = []
		for var in vars:
			try:
				if not self.global_store.resources[var.resource_name].writable:
					unwritable_resources.append(var.resource_name)
			except KeyError:
				missing_resources.append(var.resource_name)

		if missing_resources:
			MessageDialog(self, ', '.join(missing_resources), 'Missing resources').Show()
		if unwritable_resources:
			MessageDialog(self, ', '.join(unwritable_resources), 'Unwritable resources').Show()
		if missing_resources or unwritable_resources:
			return None

		return vars

	def reset(self, from_zero):
		vars = self.choose_variables()
		if vars is None:
			return

		self.to_button.Disable()
		self.from_button.Disable()

		def exception_callback(e):
			MessageDialog(self, str(e), 'Error writing to resource').Show()

		def sweep_all_vars():
			try:
				thrs = []
				for var in vars:
					resource = self.global_store.resources[var.resource_name]

					if from_zero:
						value_from, value_to = 0, var.with_type(var.const)
					else:
						value_from, value_to = var.with_type(var.const), 0

					thr = Thread(target=resource.sweep, args=(value_from, value_to, self.reset_steps_input.Value),
							kwargs={'exception_callback': partial(wx.CallAfter, exception_callback)})
					thr.daemon = True
					thrs.append(thr)

				for thr in thrs:
					thr.start()
				for thr in thrs:
					thr.join()
			finally:
				if self:
					wx.CallAfter(self.to_button.Enable)
					wx.CallAfter(self.from_button.Enable)

		thr = Thread(target=sweep_all_vars)
		thr.daemon = True
		thr.start()

	def OnResetToZero(self, evt=None):
		self.reset(False)

	def OnResetFromZero(self, evt=None):
		self.reset(True)



from functools import wraps
from itertools import chain
from numpy import linspace, meshgrid, sort, unique
from scipy.interpolate import griddata

"""
Generic tools.
"""


def flatten(iterable):
	"""
	Flatten an iterable by one level.
	"""

	return chain.from_iterable(iterable)


def sift(items, cls):
	"""
	Filter out items which are not instances of cls.
	"""

	return [item for item in items if isinstance(item, cls)]


def triples_to_mesh(x, y, z):
	"""
	Convert 3 equal-sized lists of co-ordinates into an interpolated 2D mesh of z-values.

	Returns a tuple of:
		the mesh
		the x bounds
		the y bounds
		the z bounds
	"""

	x_values, y_values = sort(unique(x)), sort(unique(y))

	x_space = linspace(x_values[0], x_values[-1], len(x_values))
	y_space = linspace(y_values[0], y_values[-1], len(y_values))

	target_x, target_y = meshgrid(x_space, y_space)

	target_z = griddata((x, y), z, (target_x, target_y), method='cubic')

	return (target_z, (x_values[0], x_values[-1]), (y_values[0], y_values[-1]),
			(min(z), max(z)))


class Enum(set):
	"""
	An enumerated type.

	>>> e = Enum(['a', 'b', 'c'])
	>>> e.a
	'a'
	>>> e.d
	...
	AttributeError: 'Enum' object has no attribute 'd'
	"""

	def __getattribute__(self, name):
		if name in self:
			return name
		else:
			return set.__getattribute__(self, name)


class PubDict(dict):
	"""
	A locking, publishing dictionary.
	"""

	def __init__(self, lock, pub, topic, *args, **kwargs):
		"""
		lock: A re-entrant lock which supports context management.
		pub: A PubSub publisher.
		topic: The topic on which to send messages.
		"""

		dict.__init__(self, *args, **kwargs)

		self.lock = lock
		self.pub = pub
		self.topic = topic

	def __setitem__(self, k, v):
		"""
		Note: Values cannot be overwritten, to ensure that removal is always handled explicitly.
		"""

		with self.lock:
			if k in self:
				raise KeyError(k)

			if v is None:
				raise ValueError('No value given.')

			dict.__setitem__(self, k, v)

			self.pub.sendMessage('{0}.added'.format(self.topic), name=k, value=v)

	def __delitem__(self, k):
		with self.lock:
			dict.__delitem__(self, k)

			self.pub.sendMessage('{0}.removed'.format(self.topic), name=k)


class Synchronized(object):
	"""
	A decorator for methods which must be synchronized within an object instance.
	"""

	@staticmethod
	def __call__(f):
		@wraps(f)
		def decorated(self, *args, **kwargs):
			with self.lock:
				return f(self, *args, **kwargs)

		return decorated


class Without(object):
	"""
	A no-op object for use with "with".
	"""

	def __enter__(self, *args, **kwargs):
		return None

	def __exit__(self, *args, **kwargs):
		return False



from nose.tools import eq_
from unittest import main, TestCase

from spacq.tests.tool.box import AssertHandler

from .. import tools


class StrToBoolTest(TestCase):
	def testConversion(self):
		"""
		Try some simple cases.
		"""

		eq_(tools.str_to_bool(''), False)
		eq_(tools.str_to_bool('False'), False)
		eq_(tools.str_to_bool('false'), False)

		eq_(tools.str_to_bool('True'), True)
		eq_(tools.str_to_bool('123'), True)
		eq_(tools.str_to_bool('Anything'), True)
		eq_(tools.str_to_bool('else!'), True)


class BlockDataTest(TestCase):
	def testToAndFromBlockData(self):
		"""
		Routine conversions.
		"""

		binary_data = ''.join([chr(x) for x in xrange(256)])

		data = [
			('', '#10'),
			(' ', '#11 '),
			('something longer', '#216something longer'),
			('and binary data: ' + binary_data, '#3273and binary data: ' + binary_data)
		]

		for d, b in data:
			eq_(tools.BlockData.to_block_data(d), b)
			eq_(tools.BlockData.from_block_data(b), d)

	def testFromIndefiniteBlockData(self):
		"""
		Indefinite inputs.
		"""

		binary_data = ''.join([chr(x) for x in xrange(256)])

		data = [
			('', '#0\n'),
			(' ', '#0 \n'),
			('something longer', '#0something longer\n'),
			('and binary data: ' + binary_data, '#0and binary data: ' + binary_data + '\n')
		]

		for d, b in data:
			eq_(tools.BlockData.from_block_data(b), d)

	def testFromSlightlyBadData(self):
		"""
		Not valid, but parsable inputs.
		"""

		data = [
			('Too ', '#14Too long.', 'extra data ignored: \'long.\''),
		]

		log = AssertHandler()

		for d, b, msg in data:
			log.flush()
			eq_(tools.BlockData.from_block_data(b), d)
			log.assert_logged('warning', msg)

	def testFromBadBlockData(self):
		"""
		Invalid inputs.
		"""

		data = ['', '123Off to a bad start!', '#', '#0', '#0non-terminated',
				'#X123', '#1', '#29', '#24test', '#44444Too short.']

		for b in data:
			try:
				tools.BlockData.from_block_data(b)
			except tools.BlockDataError:
				pass
			else:
				assert False, 'Expected BlockDataError.'


class BinaryBinaryEncoderTest(TestCase):
	def testEncodeDecode(self):
		"""
		Routine conversions.
		"""

		data = [
			('', '', {}, ''),
			('hex', '\x0e', {}, '0e'),
			('1234', '\x12\x34', {}, '1234'),
			('1234 5678 9 0 a b', '\x12\x34\x56\x78\x90\xab', {}, '1234 5678 90ab'),
			('  ABCDEF   FEDCBA   ', '\xab\xcd\xef\xfe\xdc\xba', {}, 'abcd effe dcba'),
			('0001 0203', '\x00\x01\x02\x03', {}, '0001 0203'),
			('00 010203', '\x00\x01\x02\x03', {'pair_size': 3}, '000102 03'),
			('00 01 02 03', '\x00\x01\x02\x03', {'pair_up': False}, '00010203'),
		]

		for u, e, args, d in data:
			eq_(tools.BinaryEncoder.encode(u), e)
			eq_(tools.BinaryEncoder.decode(e, **args), d)

	def testLength(self):
		"""
		Routine calculations.
		"""

		data = [
			('', 0),
			('nothing', 0),
			('0000', 2),
			('001', 2),
			('data 0101 234', 5),
			('data 0101 2345', 6),
			('data 0101 2345 6', 6),
		]

		for d, l in data:
			eq_(tools.BinaryEncoder.length(d), l)


if __name__ == '__main__':
	main()



import logging
log = logging.getLogger(__name__)

from nose.plugins.skip import SkipTest
import re
from unittest import TestCase

from testconfig import config as tc


class AssertHandler(logging.handlers.BufferingHandler):
	"""
	A logging handler that allows making assertions based on its contents.
	"""

	def __init__(self, capacity=100, *args, **kwargs):
		"""
		Add ourselves to the main logger.
		"""

		logging.handlers.BufferingHandler.__init__(self, capacity, *args, **kwargs)

		logging.getLogger().addHandler(self)

	def assert_logged(self, level, msg, ignore_case=True, literal=False):
		"""
		Assert that a message matching the level and regular expression has been logged.
		"""

		level = level.lower()

		re_flags = 0
		if ignore_case:
			re_flags |= re.IGNORECASE

		for record in self.buffer:
			if record.levelname.lower() == level:
				if (literal and msg == record.msg or
						not literal and re.search(msg, record.msg, re_flags)):
					return

		assert False, 'Log message not found at level "{0}": {1}'.format(level, msg)


class DeviceServerTestCase(TestCase):
	"""
	Class for a device server test.
	"""

	mock = False

	def obtain_device(self, impl=None, manufacturer=None, model=None):
		"""
		Try to get a handle for a physical device.
		"""

		if self.mock:
			return impl()

		all_devices = tc['devices'].values()

		if manufacturer is None or model is None:
			if impl is not None:
				return impl(**all_devices[-1]['address'])
			else:
				return all_devices[-1]

		potential_devices = [dev for dev in all_devices if 'address' in dev and
				dev['manufacturer'] == manufacturer and dev['model'] == model]

		for device in potential_devices:
			try:
				if impl is not None:
					return impl(**device['address'])
				else:
					return device
			except Exception as e:
				log.info('Could not connect to device at "{0}": {1}'.format(device['address'], e))

		raise SkipTest('Could not connect to device.')



from unittest import main

from ... import ips120_10
from .. import mock_ips120_10

from ...tests.server.test_ips120_10 import IPS120_10Test


# Don't lose the real device.
real_IPS120_10 = ips120_10.IPS120_10
is_mock = IPS120_10Test.mock


def setup():
	# Run the tests with a fake device.
	ips120_10.IPS120_10 = mock_ips120_10.MockIPS120_10
	IPS120_10Test.mock = True

def teardown():
	# Restore the real device for any remaining tests.
	ips120_10.IPS120_10 = real_IPS120_10
	IPS120_10Test.mock = is_mock


if __name__ == '__main__':
	main()



from spacq.tool.box import Enum

from .abstract_device import DeviceNotFoundError

"""
Device configuration.
"""


cached_tree = None

def device_tree():
	"""
	Build a device tree from the existing devices.
	"""

	global cached_tree

	if cached_tree is not None:
		return cached_tree

	from .. import devices

	tree = {}

	for manufacturer in devices.manufacturers:
		subtree = {}

		for model, mock_model in zip(manufacturer.models, manufacturer.mock_models):
			if model is None and mock_model is None:
				continue
			elif model is not None and mock_model is None:
				name = model.name
			elif model is None and mock_model is not None:
				name = mock_model.name
			elif model is not None and mock_model is not None:
				if model.name != mock_model.name:
					raise ValueError('Different device names: "{0}" and '
							'"{1}".'.format(model.name, mock_model.name))

				name = model.name

			subtree[name] = {}

			if model is not None:
				subtree[name]['real'] = model.implementation

			if mock_model is not None:
				subtree[name]['mock'] = mock_model.implementation

		tree[manufacturer.name] = subtree

	cached_tree = tree
	return cached_tree


class ConnectionError(Exception):
	"""
	Unable to connect.
	"""

	pass


class DeviceConfig(object):
	"""
	Description for a device.
	"""

	address_modes = Enum([
		'ethernet',
		'gpib',
		'usb',
	])

	def __init__(self, name):
		self.name = name

		# Connection configuration.
		self.address_mode = None
		self.ip_address = None
		self.gpib_board = 0
		self.gpib_pad = 0
		self.gpib_sad = 0
		self.usb_resource = None

		# Information about module that implements this device.
		self.manufacturer = None
		self.model = None
		self.mock = False

		# Device-specific GUI setup.
		self.gui_setup = None

		# Resource path to label mappings.
		self.resource_labels = {}

		# The connected device object.
		self._device = None

		# Label to resource object mappings.
		self.resources = {}

	def __getstate__(self):
		"""
		Return a modified dictionary for pickling.
		"""

		result = self.__dict__.copy()

		# Do not pickle references to mutable objects.
		del result['_device']
		del result['resources']

		return result

	def __setstate__(self, dict):
		"""
		Revert the changes done by __getstate__.
		"""

		self.__dict__ = dict

		# Set missing values to defaults.
		self._device = None
		self.resources = {}

	@property
	def device(self):
		"""
		The connected device object.
		"""

		return self._device

	@device.setter
	def device(self, value):
		self._device = value

		self.gui_setup = None
		try:
			self.gui_setup = self._device._gui_setup
		except AttributeError:
			# No device or no GUI setup available.
			pass

	def diff_resources(self, new):
		"""
		Compare the resources belonging to 2 DeviceConfig objects.

		The result is a tuple of:
			resources which appear
			resources which change
			resources which disappear
		where all "resources" are resource labels.
		"""

		old_labels, new_labels = set(self.resources), set(new.resources)

		appeared = new_labels - old_labels
		disappeared = old_labels - new_labels

		changed = set()
		possibly_changed = old_labels.intersection(new_labels)
		for p in possibly_changed:
			if self.resources[p] is not new.resources[p]:
				changed.add(p)

		return (appeared, changed, disappeared)

	def connect(self):
		"""
		Create an instance of the implementation and connect to it.
		"""

		if self.address_mode not in self.address_modes:
			raise ConnectionError('Invalid address mode specified.')

		if self.manufacturer is None or self.model is None:
			raise ConnectionError('No implementation specified.')

		address = {}

		if not self.mock:
			if self.address_mode == self.address_modes.ethernet:
				if self.ip_address is None:
					raise ConnectionError('No IP address specified.')

				address['ip_address'] = self.ip_address
			elif self.address_mode == self.address_modes.gpib:
				address['gpib_board'] = self.gpib_board
				address['gpib_pad'] = self.gpib_pad
				address['gpib_sad'] = self.gpib_sad
			elif self.address_mode == self.address_modes.usb:
				if self.usb_resource is None:
					raise ConnectionError('No USB resource specified.')

				address['usb_resource'] = self.usb_resource

		tree = device_tree()

		try:
			subtree = tree[self.manufacturer]
		except KeyError:
			raise ConnectionError('Unknown manufacturer: {0}'.format(self.manufacturer))

		try:
			subtree = subtree[self.model]
		except KeyError:
			raise ConnectionError('Unknown model: {0}'.format(self.model))

		kind = 'mock' if self.mock else 'real'

		try:
			implementation = subtree[kind]
		except KeyError:
			raise ConnectionError('Unknown kind: {0}'.format(kind))

		try:
			device = implementation(autoconnect=False, **address)
		except (ValueError, NotImplementedError) as e:
			raise ConnectionError('Unable to create device.', e)

		try:
			device.connect()
		except DeviceNotFoundError as e:
			raise ConnectionError('Unable to make connection to device.', e)

		self.device = device



import logging
log = logging.getLogger(__name__)

from spacq.tool.box import Enum

"""
Figure out which plot formats can be used on this system.
"""


# 2D curve, 3D colormapped, 3D surface, 3D waveforms
formats = Enum(['two_dimensional', 'colormapped', 'surface', 'waveforms'])


# Try to import all available formats.
available_formats = {}

try:
	from .colormapped import ColormappedPlotSetupDialog
except ImportError as e:
	log.debug('Could not import from .colormapped: {0}'.format(str(e)))
else:
	available_formats[formats.colormapped] = ColormappedPlotSetupDialog

try:
	from .surface import SurfacePlotSetupDialog, WaveformsPlotSetupDialog
except ImportError as e:
	log.debug('Could not import from .surface: {0}'.format(str(e)))
else:
	available_formats[formats.surface] = SurfacePlotSetupDialog
	available_formats[formats.waveforms] = WaveformsPlotSetupDialog

try:
	from .two_dimensional import TwoDimensionalPlotSetupDialog
except ImportError as e:
	log.debug('Could not import from .two_dimensional: {0}'.format(str(e)))
else:
	available_formats[formats.two_dimensional] = TwoDimensionalPlotSetupDialog


log.debug('Available plot formats: {0}'.format(', '.join(available_formats.keys())))



from threading import RLock
from pubsub import pub

from spacq.tool.box import PubDict

"""
A single storage location for everything globally-unique.
"""


class GlobalStore(object):
	"""
	A global value store for an entire application.
	"""

	def __init__(self):
		self.lock = RLock()

		self.devices = PubDict(self.lock, pub, 'device')
		self.resources = PubDict(self.lock, pub, 'resource')
		self.variables = PubDict(self.lock, pub, 'variable')

		self.pulse_program = None



import ObjectListView
from threading import Thread
from time import sleep
import wx

from spacq.devices.config import DeviceConfig
from ..tool.box import MessageDialog
from .device.device_config import DeviceConfigDialog

"""
An interface for creating and editing DeviceConfig objects.
"""


class DeviceColumnDefn(ObjectListView.ColumnDefn):
	"""
	A column with useful defaults.
	"""

	def __init__(self, align='left', *args, **kwargs):
		ObjectListView.ColumnDefn.__init__(self, align=align, *args, **kwargs)

		# No auto-width if space filling.
		if self.isSpaceFilling:
			self.width = 0


class DevicesPanel(wx.Panel):
	col_name = DeviceColumnDefn(title='Name', valueGetter='name', width=200)
	col_connection = DeviceColumnDefn(title='Connection', width=110,
			valueGetter=lambda x: '{0}onnected'.format('C' if x.device is not None else 'Disc'))
	col_setup = DeviceColumnDefn(title='Setup', width=70,
			valueGetter=lambda x: 'Setup...' if x.gui_setup is not None else '')
	col_status = DeviceColumnDefn(title='Status', isSpaceFilling=True, isEditable=False,
			valueGetter=lambda x: (x.device.status[0] if x.device.status else 'Idle') if x.device is not None else '')

	def __init__(self, parent, global_store, dialog_owner, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		self.global_store = global_store
		self.dialog_owner = dialog_owner

		# Panel.
		panel_box = wx.BoxSizer(wx.VERTICAL)

		## OLV.
		self.olv = ObjectListView.FastObjectListView(self)
		panel_box.Add(self.olv, proportion=1, flag=wx.ALL|wx.EXPAND)

		self.olv.SetColumns([self.col_name, self.col_connection, self.col_setup, self.col_status])
		self.olv.SetSortColumn(self.col_name)

		self.olv.cellEditMode = self.olv.CELLEDIT_DOUBLECLICK
		self.olv.Bind(ObjectListView.EVT_CELL_EDIT_STARTING, self.OnCellEditStarting)
		self.olv.Bind(ObjectListView.EVT_CELL_EDIT_FINISHING, self.OnCellEditFinishing)

		## Buttons.
		button_box = wx.BoxSizer(wx.HORIZONTAL)
		panel_box.Add(button_box, proportion=0, flag=wx.ALL|wx.CENTER)

		### Row buttons.
		row_box = wx.BoxSizer(wx.HORIZONTAL)
		button_box.Add(row_box)

		add_button = wx.Button(self, wx.ID_ADD)
		add_button.Bind(wx.EVT_BUTTON, self.OnAddDevice)
		row_box.Add(add_button)

		remove_button = wx.Button(self, wx.ID_REMOVE)
		remove_button.Bind(wx.EVT_BUTTON, self.OnRemoveDevices)
		row_box.Add(remove_button)

		self.SetMinSize((600, 250))
		self.SetSizer(panel_box)

		with self.global_store.devices.lock:
			for name, dev in self.global_store.devices.iteritems():
				self.olv.AddObject(dev)

	def update_resources(self, old, new):
		"""
		Inform everybody of updated resources.
		"""

		(appeared, changed, disappeared) = old.diff_resources(new)

		with self.global_store.resources.lock:
			# Check for conflicts.
			conflicting_resources = [label for label in appeared if label in self.global_store.resources]
			if conflicting_resources:
				return conflicting_resources

			# Set up the resources.
			for label in disappeared.union(changed):
				del self.global_store.resources[label]

			for label in appeared.union(changed):
				self.global_store.resources[label] = new.resources[label]

		return []

	def OnCellEditStarting(self, evt):
		col = evt.objectListView.columns[evt.subItemIndex]
		dev = evt.rowModel

		# Ignore frivolous requests.
		if evt.rowIndex < 0:
			evt.Veto()
			return

		veto = False

		if col == self.col_connection:
			def ok_callback(dlg):
				dev_new = dlg.GetValue()
				# Use the new instance.
				with self.global_store.devices.lock:
					del self.global_store.devices[dev.name]
					self.global_store.devices[dev_new.name] = dev_new

				conflicting_resources = self.update_resources(dev, dev_new)
				if conflicting_resources:
					MessageDialog(self, ', '.join(conflicting_resources), 'Conflicting resources').Show()

					return False

				# Close the old device as necessary.
				if dev.device is not None and dev_new.device != dev.device:
					dev.device.close()

				self.olv.RemoveObject(dev)
				self.olv.AddObject(dev_new)

				return True

			dlg = DeviceConfigDialog(self, ok_callback, title=dev.name)
			dlg.SetValue(dev)
			dlg.Show()

			veto = True
		elif col == self.col_setup:
			if dev.gui_setup is not None:
				dev.gui_setup(self.dialog_owner, self.global_store, dev.name).Show()

			veto = True

		if veto:
			# No need to use the default editor.
			evt.Veto()

	def OnCellEditFinishing(self, evt):
		col = evt.objectListView.columns[evt.subItemIndex]

		if col == self.col_name:
			dev = evt.rowModel # With old name.
			dev_new_name = evt.editor.Value

			if dev_new_name == dev.name:
				# Not actually changed.
				return

			# Attempt to add a new entry first.
			try:
				self.global_store.devices[dev_new_name] = dev
			except KeyError:
				MessageDialog(self, dev_new_name, 'Device name conflicts').Show()
				evt.Veto()
				return

			# Remove the old entry.
			del self.global_store.devices[dev.name]

	def OnAddDevice(self, evt=None):
		"""
		Add a blank variable to the OLV.
		"""

		# Ensure that we get a unique name.
		with self.global_store.devices.lock:
			num = 1
			done = False
			while not done:
				name = 'New device {0}'.format(num)
				dev_cfg = DeviceConfig(name=name)

				try:
					self.global_store.devices[name] = dev_cfg
				except KeyError:
					num += 1
				else:
					done = True

		self.olv.AddObject(dev_cfg)

		# OLV likes to select a random item at this point.
		self.olv.DeselectAll()

	def OnRemoveDevices(self, evt=None):
		"""
		Remove all selected variables from the OLV.
		"""

		selected = self.olv.GetSelectedObjects()

		connected_devices = set()
		for row in selected:
			if row.device is not None:
				connected_devices.add(row.name)

		if connected_devices:
			MessageDialog(self, ', '.join(sorted(connected_devices)), 'Devices still connected').Show()
			return

		if selected:
			self.olv.RemoveObjects(selected)

		for row in selected:
			del self.global_store.devices[row.name]


class DeviceConfigFrame(wx.Frame):
	def __init__(self, parent, global_store, close_callback, *args, **kwargs):
		wx.Frame.__init__(self, parent, title='Device Configuration', *args, **kwargs)

		self.close_callback = close_callback

		# Frame.
		frame_box = wx.BoxSizer(wx.VERTICAL)

		## Devices.
		self.devices_panel = DevicesPanel(self, global_store, parent)
		frame_box.Add(self.devices_panel, proportion=1, flag=wx.EXPAND)

		self.SetSizerAndFit(frame_box)

		self.Bind(wx.EVT_CLOSE, self.OnClose)

		thr = Thread(target=self.status_poller)
		thr.daemon = True
		thr.start()

	def status_poller(self):
		"""
		Keep updating the status as long as the frame is open.
		"""

		while True:
			try:
				self.devices_panel.olv.RefreshObjects()
			except wx.PyDeadObjectError:
				# The panel has left the building.
				return

			sleep(0.2)

	def OnClose(self, evt):
		self.close_callback()

		evt.Skip()



from copy import deepcopy
from nose.tools import assert_raises, eq_
from unittest import main, TestCase

from .. import units


class QuantityTest(TestCase):
	# String value, value, units, multiplier, proper string value.
	data = [
		# Straightforward.
		('-1 s', -1, 's', 0, None),
		('0 s', 0, 's', 0, None),
		('1e0 s', 1, 's', 0, '1 s'),
		# Multiple units.
		('10 A.m', 10, 'A.m', 0, None),
		('0.5 GJ.Hz.cd', 0.5, 'GJ.Hz.cd', 9, None),
		('500 kg.ms-1', 500, 'kg.ms-1', -3, None),
		('-1234567890123 m.s-2', -1234567890123, 'm.s-2', 0, '-1.23456789e+12 m.s-2'),
		# Large & small.
		('3e40 nA', 3e40, 'nA', -9, '3e+40 nA'),
		('-3e40 mA', -3e40, 'mA', -3, '-3e+40 mA'),
		('7e-40 GA', 7e-40, 'GA', 9, None),
		('-7e-40 uA', -7e-40, 'uA', -6, None),
		# Various whitespace.
		('5s', 5, 's', 0, '5 s'),
		('5s . Hz\t2 ', 5, 's.Hz2', 0, '5 s.Hz2'),
		(' \t 123454321 \t   uHz  \t  ', 123454321, 'uHz', -6, '123454321 uHz'),
	]

	def testSimple(self):
		"""
		Some simple quantities.
		"""

		for string, value, orig_units, multiplier, _ in self.data:
			q = units.Quantity(string)

			eq_(q, units.Quantity(value, orig_units))
			eq_(q.value, value * 10 ** multiplier)
			eq_(q.original_value, value)

	def testBadStrings(self):
		"""
		Invalid strings pretending to be quantities.
		"""

		data = [
			'', '0', 's', '0seconds', '0 something', '1234 anything', 's 0',
		]

		for string in data:
			assert_raises(ValueError, units.Quantity, string)

	def testMismatchedUnits(self):
		"""
		Values should be comparable as long as they have the same dimensions.
		"""

		qs = [
			units.Quantity(1, 'GJ.s'),
			units.Quantity(1e9, 'J.Hz-1'),
			units.Quantity(1, 'Tg.m2.s-1'),
			units.Quantity(1, 'kg.Gm.m.Hz'),
			units.Quantity(1000, 'Gg.m2.s.Hz2'),
		]

		for q in qs:
			eq_(q, qs[0])

	def testAmbiguousUnit(self):
		"""
		This shouldn't happen, but let's make sure it works.
		"""

		# Ensure it works normally.
		units.Quantity(5, 'ps')

		# Insert fake unit.
		assert 'ps' not in units.SIValues.units
		units.SIValues.units.add('ps')

		try:
			assert_raises(ValueError, units.Quantity, 5, 'ps')
		finally:
			units.SIValues.units.remove('ps')

	def testAssertDimensions(self):
		"""
		Ensure that dimensions are asserted correctly.
		"""

		q = units.Quantity(5, 's')

		# No exception.
		assert q.assert_dimensions('s', exception=False)
		assert not q.assert_dimensions('N.m', exception=False)
		assert q.assert_dimensions(q, exception=False)

		# Exception.
		assert q.assert_dimensions('s')
		assert_raises(units.IncompatibleDimensions, q.assert_dimensions, 'A')
		assert_raises(units.IncompatibleDimensions, eq_, units.Quantity(1, 's'), units.Quantity(1, 'm'))

	def testComparison(self):
		"""
		Check that comparison works.
		"""

		assert units.Quantity(1, 's.m') == units.Quantity(1, 'm.s')
		assert units.Quantity(1, 'm.s-1') < units.Quantity(2, 's-1.m')
		assert units.Quantity(5.5, 's') >= units.Quantity(5.5, 's')
		assert units.Quantity(5.5, 's') != units.Quantity(5.4, 's')

		try:
			units.Quantity(1, 's') > units.Quantity(1, 's2')
		except units.IncompatibleDimensions:
			pass
		else:
			assert False, 'Expected IncompatibleDimensions.'

		try:
			units.Quantity(1, 's') <= units.Quantity(1, 'Hz')
		except units.IncompatibleDimensions:
			pass
		else:
			assert False, 'Expected IncompatibleDimensions.'

		try:
			1 == units.Quantity(1, 's')
		except TypeError:
			pass
		else:
			assert False, 'Expected TypeError'

		try:
			1 <= units.Quantity(1, 's')
		except TypeError:
			pass
		else:
			assert False, 'Expected TypeError'

	def testArithmetic(self):
		"""
		Perform arithmetic on quantities.
		"""

		# Absolute value.
		q1 = units.Quantity(123, 'T')
		q2 = units.Quantity(0, 'T')
		q3 = units.Quantity(-123, 'T')

		eq_(abs(q1), q1)
		eq_(abs(q2), q2)
		eq_(abs(q3), q1)

		# Addition & subtraction.
		## Matching units.
		q1 = units.Quantity(5, 's')
		q2 = units.Quantity(-4, 'ms')
		q3 = units.Quantity(4.996, 's')
		q4 = units.Quantity(5.004, 's')

		eq_(q1 + q2, q3)
		eq_(q1 - q2, q4)

		## Non-matching units.
		q2 = units.Quantity(-4, 'mHz')

		try:
			q1 + q2
		except units.IncompatibleDimensions:
			pass
		else:
			assert False, 'Expected IncompatibleDimensions'

		try:
			q1 - q2
		except units.IncompatibleDimensions:
			pass
		else:
			assert False, 'Expected IncompatibleDimensions'

		try:
			units.Quantity(1, 's') + 1
		except TypeError:
			pass
		else:
			assert False, 'Expected TypeError'

		try:
			units.Quantity(1, 's') - 1
		except TypeError:
			pass
		else:
			assert False, 'Expected TypeError'

		# Multiplication by real.
		q = units.Quantity(1.5, 'N.m')

		q_mul = 2 * q
		q_mul = q_mul * 3

		eq_(q, units.Quantity(1.5, 'J'))
		eq_(q_mul, units.Quantity(9, 'J'))

		# Division by real.
		q = units.Quantity(-1.5, 'N.m')

		q_mul = q / -3

		eq_(q, units.Quantity(-1.5, 'J'))
		eq_(q_mul, units.Quantity(0.5, 'J'))

	def testRepr(self):
		"""
		Ensure that repr() gives a useful value.
		"""

		# For eval().
		Quantity = units.Quantity

		for _, value, orig_units, _, _ in self.data:
			q = units.Quantity(value, orig_units)

			eq_(eval(repr(q)), q)

	def testStr(self):
		"""
		Ensure that str() gives a meaningful value.
		"""

		for string, value, orig_units, _, proper_string in self.data:
			if proper_string is None:
				proper_string = string

			q = units.Quantity(value, orig_units)

			eq_(str(q), proper_string)

	def testDeepCopy(self):
		"""
		"""

		q = units.Quantity('100 ns.V2')

		eq_(deepcopy(q), units.Quantity('100 ns.V2'))


if __name__ == '__main__':
	main()



from functools import partial
import wx

from ...tool.box import Dialog, MessageDialog

"""
Graphical interface for data filters.
"""


class FilterEditDialog(Dialog):
	def __init__(self, parent, headings, ok_callback, *args, **kwargs):
		kwargs['style'] = kwargs.get('style', wx.DEFAULT_DIALOG_STYLE) | wx.RESIZE_BORDER
		kwargs['title'] = kwargs.get('title', 'Add filter')

		Dialog.__init__(self, parent, *args, **kwargs)

		self.ok_callback = ok_callback

		# Dialog.
		dialog_box = wx.BoxSizer(wx.VERTICAL)

		## Inputs.
		input_sizer = wx.FlexGridSizer(rows=2, cols=2, hgap=5)
		input_sizer.AddGrowableCol(1, 1)
		dialog_box.Add(input_sizer, proportion=1, flag=wx.EXPAND|wx.ALL, border=5)

		input_sizer.Add(wx.StaticText(self, label='Column:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.column_input = wx.Choice(self, choices=headings)
		input_sizer.Add(self.column_input, flag=wx.EXPAND)

		input_sizer.Add(wx.StaticText(self, label='Function:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.function_input = wx.TextCtrl(self)
		self.function_input.SetMinSize((300, -1))
		input_sizer.Add(self.function_input, flag=wx.EXPAND)

		## Buttons.
		button_box = wx.BoxSizer(wx.HORIZONTAL)
		dialog_box.Add(button_box, flag=wx.CENTER|wx.ALL, border=5)

		ok_button = wx.Button(self, wx.ID_OK)
		self.Bind(wx.EVT_BUTTON, self.OnOk, ok_button)
		button_box.Add(ok_button)

		cancel_button = wx.Button(self, wx.ID_CANCEL)
		button_box.Add(cancel_button)

		self.SetSizerAndFit(dialog_box)

	def GetValue(self):
		return (self.column_input.StringSelection, self.function_input.Value)

	def SetValue(self, values):
		(self.column_input.StringSelection, self.function_input.Value) = values

	def OnOk(self, evt=None):
		try:
			self.ok_callback(self)
		except ValueError as e:
			MessageDialog(self, str(e), 'Invalid value').Show()
			return

		self.Destroy()


class FilterListDialog(Dialog):
	def __init__(self, parent, table, close_callback, filters=None, filter_columns=None,
			*args, **kwargs):
		kwargs['style'] = kwargs.get('style', wx.DEFAULT_DIALOG_STYLE) | wx.RESIZE_BORDER
		kwargs['title'] = kwargs.get('title', 'Filters')

		Dialog.__init__(self, parent, *args, **kwargs)

		self.table = table
		self.close_callback = close_callback

		if filters is None or filter_columns is None:
			self.filters, self.filter_columns = {}, {}
		else:
			self.filters, self.filter_columns = filters, filter_columns

		# Dialog.
		dialog_box = wx.BoxSizer(wx.VERTICAL)

		## Filter list.
		self.filter_list = wx.ListBox(self, choices=self.filters.keys())
		self.filter_list.SetMinSize((100, 200))
		self.Bind(wx.EVT_LISTBOX_DCLICK, self.OnEditFilter, self.filter_list)
		dialog_box.Add(self.filter_list, proportion=1, flag=wx.EXPAND|wx.ALL, border=5)

		## Buttons.
		button_box = wx.BoxSizer(wx.HORIZONTAL)
		dialog_box.Add(button_box, flag=wx.CENTER|wx.ALL, border=5)

		add_button = wx.Button(self, wx.ID_ADD)
		add_button.Bind(wx.EVT_BUTTON, self.OnAddFilter)
		button_box.Add(add_button, flag=wx.CENTER)

		remove_button = wx.Button(self, wx.ID_REMOVE)
		remove_button.Bind(wx.EVT_BUTTON, self.OnRemoveFilter)
		button_box.Add(remove_button, flag=wx.CENTER)

		self.SetSizerAndFit(dialog_box)

		self.Bind(wx.EVT_CLOSE, self.OnClose)

	def create_filter(self, f_text, col):
		"""
		Create a filter out of text.
		"""

		col_idx = self.table.headings.index(col)

		return lambda i, x: eval(f_text.replace('x', 'float(x[{0}])'.format(col_idx)))

	@property
	def meta_filter(self):
		"""
		Create a meta-filter out of all the filters.
		"""

		filters = [self.create_filter(f, col) for f, col in
				zip(self.filters.values(), self.filter_columns.values())]

		return lambda i, x: all([f(i, x) for f in filters])

	def edit_ok_callback(self, dlg, selection=None):
		col, f = dlg.GetValue()

		name = '{0}: {1}'.format(col, f)

		if selection is not None and name == selection:
			return

		if name in self.filters:
			raise ValueError('Filter "{0}" already exists'.format(name))

		if not f:
			raise ValueError('No function provided')

		f_function = self.create_filter(f, col)

		try:
			self.table.apply_filter(f_function)
		except Exception as e:
			raise ValueError(e)

		if selection is not None:
			self.OnRemoveFilter(selection=selection)
			self.table.apply_filter(f_function)

		self.filters[name] = f
		self.filter_columns[name] = col
		self.filter_list.Items += [name]

	def OnAddFilter(self, evt=None):
		FilterEditDialog(self, self.table.headings, self.edit_ok_callback).Show()

	def OnEditFilter(self, evt=None):
		selection = self.filter_list.StringSelection

		if not selection:
			return

		dlg = FilterEditDialog(self, self.table.headings, partial(self.edit_ok_callback, selection=selection),
				title='Edit filter')
		dlg.SetValue((self.filter_columns[selection], self.filters[selection]))
		dlg.Show()

	def OnRemoveFilter(self, evt=None, selection=None):
		if selection is None:
			selection = self.filter_list.StringSelection

		if not selection:
			return

		try:
			del self.filters[selection]
		except KeyError:
			pass

		try:
			del self.filter_columns[selection]
		except KeyError:
			pass

		self.filter_list.Items = [x for x in self.filter_list.Items if x != selection]

		self.table.apply_filter(self.meta_filter, afresh=True)

	def OnClose(self, evt):
		self.close_callback(self)

		evt.Skip()



from nose.tools import assert_raises, eq_
from numpy.testing import assert_array_almost_equal
from os import path
from unittest import main, TestCase

from ...units import Quantity
from ..parser import Parser

from .. import tree


resource_dir = path.join(path.dirname(__file__), 'resources')


class ValidTreeTest(TestCase):
	prog = Parser()("""
		int repeat = 2
		delay abc1 = 100 ns, def2
		pulse ghi1, jkl2 = {{shape: 'square'}}
		output mno1, pqr2

		ghi1.shape = '{0}'
		ghi1.length = 8 ns
		ghi1.amplitude = -1.0 mV

		abc1
		(10 ns):pqr2

		times repeat {{
			def2
			ghi1:mno1
			(ghi1 abc1 10 ns jkl2):mno1 (def2 jkl2 def2):pqr2
		}}

		acquire

		5 ns
	""".format(path.join(resource_dir, 'non-square')))

	def testDeclarations(self):
		env = tree.Environment()

		env.stage = env.stages.declarations
		env.traverse_tree(self.prog)

		expected = {
			'_acq_marker': 'acq_marker',
			'abc1': 'delay',
			'def2': 'delay',
			'ghi1': 'pulse',
			'jkl2': 'pulse',
			'mno1': 'output',
			'pqr2': 'output',
			'repeat': 'int',
		}

		eq_(env.errors, [])
		eq_(env.variables, expected)

	def testValues(self):
		env = tree.Environment()

		env.stage = env.stages.declarations
		env.traverse_tree(self.prog)

		env.stage = env.stages.values
		env.traverse_tree(self.prog)

		expected = {
			('abc1',): Quantity('100 ns'),
			('ghi1', 'amplitude'): Quantity(-1, 'mV'),
			('ghi1', 'length'): Quantity(8, 'ns'),
			('ghi1', 'shape'): path.join(resource_dir, 'non-square'),
			('jkl2', 'shape'): 'square',
			('repeat',): 2,
		}

		missing = set([('_acq_marker', 'marker_num'), ('_acq_marker', 'output'), ('def2',),
				('jkl2', 'amplitude'), ('jkl2', 'length')])

		eq_(env.errors, [])
		eq_(env.values, expected)
		eq_(env.missing_values, missing)

		# Later additions (perhaps from the UI).
		updates = {
			('_acq_marker', 'marker_num'): 1,
			('_acq_marker', 'output'): 'mno1',
			('def2',): Quantity('7 ns'),
			('jkl2', 'amplitude'): Quantity(1, 'V'),
			('jkl2', 'length'): Quantity(50, 'ns'),
		}

		for name, value in updates.items():
			env.set_value(name, value)

		expected.update(updates)

		eq_(env.errors, [])
		eq_(env.values, expected)

	def testCommands(self):
		env = tree.Environment()

		env.stage = env.stages.declarations
		env.traverse_tree(self.prog)

		env.stage = env.stages.commands
		env.traverse_tree(self.prog)

		eq_(env.errors, [])
		assert env.acquisition

	def testWaveforms(self):
		env = tree.Environment()

		env.stage = env.stages.declarations
		env.traverse_tree(self.prog)

		env.stage = env.stages.values
		env.traverse_tree(self.prog)

		env.stage = env.stages.commands
		env.traverse_tree(self.prog)

		updates = {
			('_acq_marker', 'marker_num'): 5,
			('_acq_marker', 'output'): 'pqr2',
			('def2',): Quantity(7, 'ns'),
			('jkl2', 'amplitude'): Quantity(1, 'V'),
			('jkl2', 'length'): Quantity(50, 'ns'),
		}

		for name, value in updates.items():
			env.set_value(name, value)

		env.frequency = Quantity(1, 'GHz')
		env.stage = env.stages.waveforms
		env.traverse_tree(self.prog)

		eq_(env.errors, [])

		mno1 = env.waveforms['mno1']
		mno1_gen = env.generators['mno1']
		non_square = mno1_gen._scale_waveform([0.1, 0.5, 0.7, 1.0] + [4.2] * 3 + [3.6, 9.9],
				Quantity(-1, 'mV').value, Quantity(8, 'ns'))
		loop = [0.0] * 7 + non_square * 2 + [0.0] * 110 + [1.0] * 50 + [0.0]
		assert_array_almost_equal(mno1.data, [0.0] * 110 + loop * 2 + [0.0] * 5)

		pqr2 = env.waveforms['pqr2']
		loop = [0.0] * 22 + [1.0] * 50 + [0.0] * 112
		assert_array_almost_equal(pqr2.data, [0.0] * 110 + loop * 2 + [0.0] * 5)

		assert 1 not in pqr2.markers
		eq_(pqr2.markers[5], [False] * 478 + [True] * 5)


class InvalidTreeTest(TestCase):
	def testDeclarations(self):
		prog = Parser()("""
			int repeat = 2
			delay abc1 = 100 ns, def2
			pulse ghi1, jkl2 = {shape: 'square'}
			output mno1, pqr2

			delay abc1, def2 = 10 ms, stu3

			vwx4

			times 5 {
				int x = 9
			}
		""")

		env = tree.Environment()

		env.stage = env.stages.declarations
		env.traverse_tree(prog)

		expected_errors = ['Re-decl'] * 2 + ['Declara']

		eq_(len(env.errors), len(expected_errors))

		for error, expected_error in zip(env.errors, expected_errors):
			assert error[0].startswith(expected_error), error

	def testValues(self):
		prog = Parser()("""
			int repeat = 2
			delay abc1 = 100 ns, def2
			pulse ghi1, jkl2 = {shape: 'square'}
			output mno1, pqr2

			delay abc1 = 50 ms, def2
			mno1 = "test"
			def2 = 6 ; def2 = 6 Hz
			ghi1 = 0
			ghi1.shape = 50 ms
			jkl2.amplitude = 8
			jkl2.length = 1234 A
			repeat = 6 s
			repeat = 2.0
			zzz1 = 5 ms
			zzz1.foo = 5 ms
			ghi1.something_else = 5

			times 5 {
				int x = 9
				y = 10
			}
		""")

		env = tree.Environment()

		env.stage = env.stages.declarations
		env.traverse_tree(prog)

		env.errors = []
		env.stage = env.stages.values
		env.traverse_tree(prog)

		expected_errors = (['Re-assi', 'Cannot a'] + ['Must assi'] * 8 + ['Undecla', 'Unrec'] * 2 +
				['Assig'] + ['Undecl'])

		eq_(len(env.errors), len(expected_errors))

		for error, expected_error in zip(env.errors, expected_errors):
			assert error[0].startswith(expected_error), (error, expected_error)

		assert_raises(KeyError, env.set_value, ('xyz',), 'zyx')

	def testCommands(self):
		prog = Parser()("""
			int repeat = 2
			delay abc1 = 100 ns, def2
			pulse ghi1, jkl2 = {shape: 'square'}
			output mno1, pqr2

			abc1
			(10 ns):pqr2

			times repeat {
				def2
				ghi1:mno1
				(ghi1 abc1 10 ns jkl2):mno1 (10 V def2 jkl2 def2 pqr2):pqr2

				acquire
				mno1
			}

			acquire

			times ghi1 {}

			acquire

			5 ns
			1 A
		""")
		env = tree.Environment()

		env.stage = env.stages.declarations
		env.traverse_tree(prog)

		env.errors = []
		env.stage = env.stages.commands
		env.traverse_tree(prog)

		expected_errors = ['Delay mu', 'Invali', 'Not a d', 'Repeate', 'Repeti', 'Repeate', 'Delay mu']

		eq_(len(env.errors), len(expected_errors))

		for error, expected_error in zip(env.errors, expected_errors):
			assert error[0].startswith(expected_error), (error, expected_error)

	def testWaveforms(self):
		prog = Parser()("""
			delay d1

			d1
		""")
		env = tree.Environment()

		env.stage = env.stages.declarations
		env.traverse_tree(prog)

		env.stage = env.stages.values
		env.traverse_tree(prog)

		env.stage = env.stages.commands
		env.traverse_tree(prog)

		env.errors = []
		env.stage = env.stages.waveforms

		assert_raises(ValueError, env.traverse_tree, prog)


class DrawThingTest(TestCase):
	def testDrawTree(self):
		"""
		Try our hand at drawing.
		"""

		prog = Parser()("""
			pulse p1 = {shape: 'something'}
			output f1

			p1.length = 5 ns

			times 5 {
				10 ns
				p1:f1
			}

			acquire
		""")

		drawing = prog.draw()

		eq_(drawing, """\
Block
 Declaration
  pulse
  Assignment
   p1
   Dictionary
    'shape': 'something'
 Declaration
  output
  Variable
   f1
 Assignment
  Attribute
   p1
   length
  5 ns
 Loop
  5
  Block
   Delay
    10 ns
   ParallelPulses
    Pulse
     PulseSequence
      p1
     f1
  Acquire
""")


if __name__ == '__main__':
	main()



import logging
log = logging.getLogger(__name__)


name = 'Tektronix'

from . import awg5014b, dpo7104
models = [awg5014b, dpo7104]
log.debug('Found models for "{0}": {1}'.format(name, ''.join(str(x) for x in models)))

from .mock import mock_awg5014b, mock_dpo7104
mock_models = [mock_awg5014b, mock_dpo7104]
log.debug('Found mock models for "{0}": {1}'.format(name, ''.join(str(x) for x in mock_models)))



import wx
from wx.lib.masked.ipaddrctrl import IpAddrCtrl

from spacq.devices.config import device_tree, ConnectionError, DeviceConfig

from ...tool.box import load_pickled, save_pickled, Dialog, MessageDialog
from .resource_tree import DeviceResourcesPanel

"""
Device configuration for and through the GUI.
"""


class DeviceConfigPanel(wx.Panel):
	"""
	Set up a device for consumption of its resources.
	"""

	def __init__(self, parent, connection_callback=None, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		self.connection_callback = connection_callback

		# Implementation info.
		## Find all the available devices.
		self.device_tree = device_tree()
		self.manufacturers = [''] + sorted(self.device_tree.keys())
		self.models = ['']

		## Chosen values.
		self.manufacturer = None
		self.model = None

		# Panel.
		panel_box = wx.BoxSizer(wx.VERTICAL)

		## Address.
		address_static_box = wx.StaticBox(self, label='Address')
		address_box = wx.StaticBoxSizer(address_static_box, wx.VERTICAL)
		address_sizer = wx.BoxSizer(wx.HORIZONTAL)
		address_box.Add(address_sizer, flag=wx.EXPAND)
		panel_box.Add(address_box, flag=wx.EXPAND|wx.ALL, border=5)

		### Ethernet.
		ethernet_static_box = wx.StaticBox(self)
		ethernet_box = wx.StaticBoxSizer(ethernet_static_box, wx.VERTICAL)
		address_sizer.Add(ethernet_box, proportion=1)

		self.address_mode_eth = wx.RadioButton(self, label='Ethernet', style=wx.RB_GROUP)
		ethernet_box.Add(self.address_mode_eth)

		ethernet_sizer = wx.FlexGridSizer(rows=2, cols=2, hgap=5)
		ethernet_box.Add(ethernet_sizer, flag=wx.EXPAND)

		ethernet_sizer.Add(wx.StaticText(self, label='IP address:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.ip_address_input = IpAddrCtrl(self)
		ethernet_sizer.Add(self.ip_address_input, flag=wx.CENTER)

		### GPIB.
		self.gpib_static_box = wx.StaticBox(self)
		gpib_box = wx.StaticBoxSizer(self.gpib_static_box, wx.VERTICAL)
		address_sizer.Add(gpib_box, proportion=1)

		self.address_mode_gpib = wx.RadioButton(self, label='GPIB')
		gpib_box.Add(self.address_mode_gpib)

		gpib_sizer = wx.FlexGridSizer(rows=3, cols=2, hgap=5)
		gpib_box.Add(gpib_sizer, flag=wx.EXPAND)

		gpib_sizer.Add(wx.StaticText(self, label='Board:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.gpib_board_input = wx.SpinCtrl(self, min=0, max=100, initial=0)
		gpib_sizer.Add(self.gpib_board_input, flag=wx.CENTER)

		gpib_sizer.Add(wx.StaticText(self, label='PAD:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.gpib_pad_input = wx.SpinCtrl(self, min=1, max=30, initial=1)
		gpib_sizer.Add(self.gpib_pad_input, flag=wx.CENTER)

		gpib_sizer.Add(wx.StaticText(self, label='SAD:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.gpib_sad_input = wx.SpinCtrl(self, min=0, max=30, initial=0)
		gpib_sizer.Add(self.gpib_sad_input, flag=wx.CENTER)

		### USB.
		usb_static_box = wx.StaticBox(self)
		usb_box = wx.StaticBoxSizer(usb_static_box, wx.VERTICAL)
		address_box.Add(usb_box, flag=wx.EXPAND)

		self.address_mode_usb = wx.RadioButton(self, label='USB')
		usb_box.Add(self.address_mode_usb)

		usb_sizer = wx.BoxSizer(wx.HORIZONTAL)
		usb_box.Add(usb_sizer, flag=wx.EXPAND)

		usb_sizer.Add(wx.StaticText(self, label='USB resource: '),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.usb_resource_input = wx.TextCtrl(self, size=(300, -1))
		usb_sizer.Add(self.usb_resource_input, proportion=1)

		## Implementation.
		implementation_static_box = wx.StaticBox(self, label='Implementation')
		implementation_box = wx.StaticBoxSizer(implementation_static_box, wx.HORIZONTAL)
		panel_box.Add(implementation_box, flag=wx.EXPAND|wx.ALL, border=5)

		self.manufacturer_input = wx.Choice(self, choices=self.manufacturers)
		self.Bind(wx.EVT_CHOICE, self.OnManufacturer, self.manufacturer_input)
		implementation_box.Add(self.manufacturer_input, proportion=1)

		self.model_input = wx.Choice(self, choices=self.models)
		self.Bind(wx.EVT_CHOICE, self.OnModel, self.model_input)
		implementation_box.Add(self.model_input, proportion=1)

		self.mock_input = wx.CheckBox(self, label='Mock')
		implementation_box.Add(self.mock_input, flag=wx.CENTER)

		## Connection buttons.
		button_box = wx.BoxSizer(wx.HORIZONTAL)
		panel_box.Add(button_box, flag=wx.CENTER|wx.ALL, border=5)

		self.connect_button = wx.Button(self, label='Connect')
		self.Bind(wx.EVT_BUTTON, self.OnConnect, self.connect_button)
		button_box.Add(self.connect_button)

		self.disconnect_button = wx.Button(self, label='Disconnect')
		self.Bind(wx.EVT_BUTTON, self.OnDisconnect, self.disconnect_button)
		button_box.Add(self.disconnect_button)

		self.SetSizerAndFit(panel_box)

	def get_address_mode(self):
		if self.address_mode_eth.Value:
			return DeviceConfig.address_modes.ethernet
		elif self.address_mode_gpib.Value:
			return DeviceConfig.address_modes.gpib
		elif self.address_mode_usb.Value:
			return DeviceConfig.address_modes.usb

	def GetValue(self):
		dev_cfg = DeviceConfig(name=self.name)

		# Address mode.
		dev_cfg.address_mode = self.get_address_mode()

		## Ethernet.
		possible_address = self.ip_address_input.GetAddress()
		if self.ip_address_input.IsValid() and len(possible_address) > 6:
			dev_cfg.ip_address = possible_address
		else:
			dev_cfg.ip_address = None

		## GPIB.
		dev_cfg.gpib_board = self.gpib_board_input.Value
		dev_cfg.gpib_pad = self.gpib_pad_input.Value
		dev_cfg.gpib_sad = self.gpib_sad_input.Value

		## USB.
		possible_resource = self.usb_resource_input.Value
		if possible_resource:
			dev_cfg.usb_resource = possible_resource
		else:
			dev_cfg.usb_resource = None

		# Implementation.
		dev_cfg.manufacturer = self.manufacturer
		dev_cfg.model = self.model
		dev_cfg.mock = self.mock_input.Value

		# Device.
		dev_cfg.device = self.device

		# Resource labels.
		dev_cfg.resource_labels = self.resource_labels

		return dev_cfg

	def SetValue(self, dev_cfg):
		self.name = dev_cfg.name

		# Address mode.
		if dev_cfg.address_mode == DeviceConfig.address_modes.ethernet:
			self.address_mode_eth.Value = True
		elif dev_cfg.address_mode == DeviceConfig.address_modes.gpib:
			self.address_mode_gpib.Value = True
		elif dev_cfg.address_mode == DeviceConfig.address_modes.usb:
			self.address_mode_usb.Value = True

		## Ethernet.
		if dev_cfg.ip_address:
			self.ip_address_input.SetValue(dev_cfg.ip_address)

		## GPIB.
		self.gpib_board_input.Value = dev_cfg.gpib_board
		self.gpib_pad_input.Value = dev_cfg.gpib_pad
		self.gpib_sad_input.Value = dev_cfg.gpib_sad

		## USB.
		if dev_cfg.usb_resource:
			self.usb_resource_input.Value = dev_cfg.usb_resource

		# Implementation.
		if dev_cfg.manufacturer is not None:
			self.manufacturer_input.StringSelection = dev_cfg.manufacturer
			self.OnManufacturer()

		if dev_cfg.model is not None:
			self.model_input.StringSelection = dev_cfg.model
			self.OnModel()

		self.mock_input.Value = dev_cfg.mock

		# Device.
		self.device = dev_cfg.device
		if self.device is not None:
			self.connect_button.Disable()
			self.disconnect_button.Enable()
		else:
			self.connect_button.Enable()
			self.disconnect_button.Disable()

		# Resource labels.
		self.resource_labels = dev_cfg.resource_labels

		if self.device is not None and self.connection_callback is not None:
			self.connection_callback(self.device, self.resource_labels)

	def OnManufacturer(self, evt=None):
		self.manufacturer = self.manufacturers[self.manufacturer_input.Selection]

		self.models = ['']
		if self.manufacturer:
			self.models.extend(self.device_tree[self.manufacturer].keys())
		else:
			self.manufacturer = None

		self.model_input.SetItems(self.models)

	def OnModel(self, evt=None):
		self.model = self.models[self.model_input.Selection]

		if self.model:
			model = self.device_tree[self.manufacturer][self.model]

			if 'real' in model and 'mock' not in model:
				self.mock_input.Value = False
				self.mock_input.Disable()
			elif 'real' not in model and 'mock' in model:
				self.mock_input.Value = True
				self.mock_input.Disable()
			else:
				self.mock_input.Enable()
		else:
			self.model = None

	def OnDisconnect(self, evt=None):
		self.device = None

		if self.connection_callback is not None:
			self.connection_callback(None, {})

		self.disconnect_button.Disable()

	def OnConnect(self, evt=None):
		dev_cfg = self.GetValue()
		try:
			dev_cfg.connect()
		except ConnectionError as e:
			MessageDialog(self, str(e), 'Connection error').Show()
			return

		self.device = dev_cfg.device

		if self.connection_callback is not None:
			self.connection_callback(self.device, self.resource_labels)

		self.connect_button.Disable()


class DeviceConfigDialog(Dialog):
	"""
	A dialog for configuring a device, including connection and resources.
	"""

	def __init__(self, parent, ok_callback, *args, **kwargs):
		Dialog.__init__(self, parent, style=wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER,
				*args, **kwargs)

		self.ok_callback = ok_callback

		# Dialog.
		dialog_box = wx.BoxSizer(wx.VERTICAL)

		## Tabs.
		self.notebook = wx.Notebook(self)
		dialog_box.Add(self.notebook, proportion=1, flag=wx.EXPAND)

		self.resources_panel = DeviceResourcesPanel(self.notebook)
		self.connection_panel = DeviceConfigPanel(self.notebook,
				connection_callback=self.resources_panel.set_device)

		self.notebook.AddPage(self.connection_panel, 'Connection')
		self.notebook.AddPage(self.resources_panel, 'Resources')

		## End buttons.
		button_box = wx.BoxSizer(wx.HORIZONTAL)
		dialog_box.Add(button_box, flag=wx.CENTER|wx.TOP, border=10)

		### OK, cancel.
		dialog_button_box = wx.BoxSizer(wx.HORIZONTAL)
		button_box.Add(dialog_button_box)

		ok_button = wx.Button(self, wx.ID_OK)
		self.Bind(wx.EVT_BUTTON, self.OnOk, ok_button)
		dialog_button_box.Add(ok_button)

		cancel_button = wx.Button(self, wx.ID_CANCEL)
		dialog_button_box.Add(cancel_button)

		### Save, load.
		save_button_box = wx.BoxSizer(wx.HORIZONTAL)
		button_box.Add(save_button_box, flag=wx.LEFT, border=20)

		save_button = wx.Button(self, wx.ID_SAVE, label='Save...')
		self.Bind(wx.EVT_BUTTON, self.OnSave, save_button)
		save_button_box.Add(save_button)

		load_button = wx.Button(self, wx.ID_OPEN, label='Load...')
		self.Bind(wx.EVT_BUTTON, self.OnLoad, load_button)
		save_button_box.Add(load_button)

		self.SetSizerAndFit(dialog_box)

	def GetValue(self):
		dev_cfg = self.connection_panel.GetValue()
		labels, resources = self.resources_panel.GetValue()

		dev_cfg.resources = resources

		# Preserve labels between device instances.
		if dev_cfg.device is not None:
			dev_cfg.resource_labels = labels

		return dev_cfg

	def SetValue(self, dev_cfg):
		self.connection_panel.SetValue(dev_cfg)
		self.resources_panel.SetValue(dev_cfg.resource_labels, dev_cfg.resources)

	def OnOk(self, evt=None):
		if self.ok_callback(self):
			self.Destroy()

	def OnSave(self, evt=None):
		try:
			save_pickled(self, self.GetValue(), extension='dev',
					file_type='Device configuration')
		except IOError as e:
			MessageDialog(self, str(e), 'Save error').Show()
			return

	def OnLoad(self, evt=None):
		try:
			value = load_pickled(self, extension='dev', file_type='Device configuration')

			try:
				if value is not None:
					value.name = self.connection_panel.name
					self.SetValue(value)
			except Exception as e:
				raise IOError('Could not set values.', e)
		except IOError as e:
			MessageDialog(self, str(e), 'Load error').Show()
			return



from nose.tools import eq_
from unittest import main, TestCase

from ..mock.mock_abstract_device import MockAbstractDevice

from .. import config


class DeviceTreeTest(TestCase):
	def testTree(self):
		"""
		Ensure the tree looks correct.
		"""

		tree = config.device_tree()

		assert 'Agilent' in tree
		assert 'Tektronix' in tree

		agi = tree['Agilent']
		tek = tree['Tektronix']

		assert '34410A' in agi
		assert 'AWG5014B' in tek

		dm = agi['34410A']
		awg = tek['AWG5014B']

		from ..agilent.dm34410a import DM34410A
		from ..agilent.mock.mock_dm34410a import MockDM34410A
		from ..tektronix.awg5014b import AWG5014B
		from ..tektronix.mock.mock_awg5014b import MockAWG5014B

		eq_(dm['real'], DM34410A)
		eq_(dm['mock'], MockDM34410A)
		eq_(awg['real'], AWG5014B)
		eq_(awg['mock'], MockAWG5014B)


class DeviceConfigTest(TestCase):
	def testMockConnect(self):
		"""
		Connect to a mock device.
		"""

		cfg = config.DeviceConfig(name='Test')

		# These values don't matter.
		cfg.address_mode = cfg.address_modes.ethernet
		cfg.ip_address = '127.0.0.1'

		tree = config.device_tree()
		cfg.manufacturer = tree.keys()[-1]
		cfg.model = tree[cfg.manufacturer].keys()[-1]
		cfg.mock = True

		cfg.connect()

		assert isinstance(cfg.device, MockAbstractDevice)

	def testInvalidConnect(self):
		"""
		Fail to connect to a non-existing device.
		"""

		cfg = config.DeviceConfig(name='Test')

		# These values don't matter.
		cfg.address_mode = cfg.address_modes.ethernet
		cfg.ip_address = '127.0.0.1'

		# No implementation.
		try:
			cfg.connect()
		except config.ConnectionError:
			pass
		else:
			assert False, 'Expected ConnectionError.'

		# Incorrect implementation.
		cfg.manufacturer = 'Fender'
		cfg.model = 'Telecaster'
		try:
			cfg.connect()
		except config.ConnectionError:
			pass
		else:
			assert False, 'Expected ConnectionError.'

	def testDiffResources(self):
		"""
		Try changing up some resources.
		"""

		cfg1 = config.DeviceConfig(name='Test 1')
		cfg2 = config.DeviceConfig(name='Test 2')
		eq_(cfg1.diff_resources(cfg2), (set(), set(), set()))

		cfg2.resources['something'] = 'new'
		eq_(cfg1.diff_resources(cfg2), (set(['something']), set(), set()))

		cfg1.resources['for'] = 'now'
		eq_(cfg1.diff_resources(cfg2), (set(['something']), set(), set(['for'])))

		cfg1.resources['something'] = 'now'
		eq_(cfg1.diff_resources(cfg2), (set(), set(['something']), set(['for'])))


if __name__ == '__main__':
	main()



#!/usr/bin/env python2

from setuptools import setup, find_packages


def included_package(p):
	return p.startswith('spacq.') or p == 'spacq'


setup(
	name='SpanishAcquisition',
	version='1.0.1',
	author='Dmitri Iouchtchenko',
	author_email='diouchtc@uwaterloo.ca',
	description='Package for interfacing with devices and building user '
			'interfaces.',
	license='BSD',
	url='http://0.github.com/SpanishAcquisition/',
	packages=[p for p in find_packages() if included_package(p)],
	classifiers=[
		'Development Status :: 5 - Production/Stable',
		'Intended Audience :: Science/Research',
		'License :: OSI Approved :: BSD License',
		'Operating System :: OS Independent',
		'Programming Language :: Python :: 2',
	],
)
